@@ -41,42 +41,51 @@ import java.io.IOException;
 import java.io.Reader;
 import java.util.HashMap;
 import java.util.Map;
+import java.util.ArrayList;
+import java.util.List;
 
 /**
  * @author <a href="mailto:trygvis@inamo.no">Trygve Laugst&oslash;l</a>
  * @author <a href="mailto:evenisse@codehaus.org">Emmanuel Venisse</a>
- * @version $Id: ModelReader.java 1075 2008-12-31 11:51:02Z hboutemy $
+ * @version $Id: ModelReader.java 1436 2010-04-14 23:04:58Z bentmann $
  */
 public class ModelReader
 {
-    private Map/*<String, String>*/ modelAttributes = new HashMap();
+    private Map<String, String> modelAttributes = new HashMap<String, String>();
 
-    private Map/*<String, Map<String, String>>*/ classAttributes = new HashMap();
+    private Map<String, Map<String, String>> classAttributes = new HashMap<String, Map<String, String>>();
 
-    private Map/*<String, Map<String, String>>*/ fieldAttributes = new HashMap();
+    private Map<String, Map<String, String>> interfaceAttributes = new HashMap<String, Map<String, String>>();
 
-    private Map/*<String, Map<String, String>>*/ associationAttributes = new HashMap();
+    private Map<String, Map<String, String>> fieldAttributes = new HashMap<String, Map<String, String>>();
 
-    public Map/*<String, String>*/ getAttributesForModel()
+    private Map<String, Map<String, String>> associationAttributes = new HashMap<String, Map<String, String>>();
+
+    public Map<String, String> getAttributesForModel()
     {
-        return (Map) modelAttributes;
+        return modelAttributes;
     }
 
-    public Map/*<String, String>*/ getAttributesForClass( ModelClass modelClass )
+    public Map<String, String> getAttributesForClass( ModelClass modelClass )
     {
-        return (Map) classAttributes.get( modelClass.getName() );
+        return classAttributes.get( modelClass.getName() );
     }
 
-    public Map/*<String, String>*/ getAttributesForField( ModelField modelField )
+    public Map<String, String> getAttributesForInterface( ModelInterface modelInterface )
     {
-        return (Map) fieldAttributes.get( modelField.getModelClass().getName() + ':' + modelField.getName() + ':'
-                                          + modelField.getVersionRange() );
+        return interfaceAttributes.get( modelInterface.getName() );
     }
 
-    public Map/*<String, String>*/ getAttributesForAssociation( ModelAssociation modelAssociation )
+    public Map<String, String> getAttributesForField( ModelField modelField )
     {
-        return (Map) associationAttributes.get( modelAssociation.getModelClass().getName() + ':'
-            + modelAssociation.getName() + ':' + modelAssociation.getVersionRange() );
+        return fieldAttributes.get( modelField.getModelClass().getName() + ':' + modelField.getName() + ':'
+            + modelField.getVersionRange() );
+    }
+
+    public Map<String, String> getAttributesForAssociation( ModelAssociation modelAssociation )
+    {
+        return associationAttributes.get( modelAssociation.getModelClass().getName() + ':' + modelAssociation.getName()
+            + ':' + modelAssociation.getVersionRange() );
     }
 
     public Model loadModel( Reader reader )
@@ -116,27 +125,27 @@ public class ModelReader
                 if ( parseBaseElement( model, parser ) )
                 {
                 }
-                else if ( parser.getName().equals( "id" ) )
+                else if ( "id".equals( parser.getName() ) )
                 {
                     model.setId( parser.nextText() );
                 }
-                else if ( parser.getName().equals( "defaults" ) )
+                else if ( "defaults".equals( parser.getName() ) )
                 {
                     parseDefaults( model, parser );
                 }
-                else if ( parser.getName().equals( "versionDefinition" ) )
+                else if ( "versionDefinition".equals( parser.getName() ) )
                 {
                     parseVersionDefinition( model, parser );
                 }
-                else if ( parser.getName().equals( "interfaces" ) )
+                else if ( "interfaces".equals( parser.getName() ) )
                 {
                     parseInterfaces( model, parser );
                 }
-                else if ( parser.getName().equals( "classes" ) )
+                else if ( "classes".equals( parser.getName() ) )
                 {
                     parseClasses( model, parser );
                 }
-                else if ( parser.getName().equals( "model" ) )
+                else if ( "model".equals( parser.getName() ) )
                 {
                     modelAttributes = getAttributes( parser );
                 }
@@ -154,17 +163,17 @@ public class ModelReader
     {
         while ( parser.nextTag() == XmlPullParser.START_TAG )
         {
-            if ( parser.getName().equals( "default" ) )
+            if ( "default".equals( parser.getName() ) )
             {
                 ModelDefault modelDefault = new ModelDefault();
 
                 while ( parser.nextTag() == XmlPullParser.START_TAG )
                 {
-                    if ( parser.getName().equals( "key" ) )
+                    if ( "key".equals( parser.getName() ) )
                     {
                         modelDefault.setKey( parser.nextText() );
                     }
-                    else if ( parser.getName().equals( "value" ) )
+                    else if ( "value".equals( parser.getName() ) )
                     {
                         modelDefault.setValue( parser.nextText() );
                     }
@@ -215,24 +224,26 @@ public class ModelReader
     {
         while ( parser.nextTag() == XmlPullParser.START_TAG )
         {
-            if ( parser.getName().equals( "interface" ) )
+            if ( "interface".equals( parser.getName() ) )
             {
                 ModelInterface modelInterface = new ModelInterface();
 
+                Map<String, String> attributes = getAttributes( parser );
+
                 while ( parser.nextTag() == XmlPullParser.START_TAG )
                 {
                     if ( parseBaseElement( modelInterface, parser ) )
                     {
                     }
-                    else if ( parser.getName().equals( "superInterface" ) )
+                    else if ( "superInterface".equals( parser.getName() ) )
                     {
                         modelInterface.setSuperInterface( parser.nextText() );
                     }
-                    else if ( parser.getName().equals( "packageName" ) )
+                    else if ( "packageName".equals( parser.getName() ) )
                     {
                         modelInterface.setPackageName( parser.nextText() );
                     }
-                    else if ( parser.getName().equals( "codeSegments" ) )
+                    else if ( "codeSegments".equals( parser.getName() ) )
                     {
                         parseCodeSegment( modelInterface, parser );
                     }
@@ -243,6 +254,7 @@ public class ModelReader
                 }
 
                 model.addInterface( modelInterface );
+                interfaceAttributes.put( modelInterface.getName(), attributes );
             }
             else
             {
@@ -256,34 +268,34 @@ public class ModelReader
     {
         while ( parser.nextTag() == XmlPullParser.START_TAG )
         {
-            if ( parser.getName().equals( "class" ) )
+            if ( "class".equals( parser.getName() ) )
             {
                 ModelClass modelClass = new ModelClass();
 
-                Map/*<String, String>*/ attributes = getAttributes( parser );
+                Map<String, String> attributes = getAttributes( parser );
 
                 while ( parser.nextTag() == XmlPullParser.START_TAG )
                 {
                     if ( parseBaseElement( modelClass, parser ) )
                     {
                     }
-                    else if ( parser.getName().equals( "interfaces" ) )
+                    else if ( "interfaces".equals( parser.getName() ) )
                     {
                         parseClassInterfaces( modelClass, parser );
                     }
-                    else if ( parser.getName().equals( "superClass" ) )
+                    else if ( "superClass".equals( parser.getName() ) )
                     {
                         modelClass.setSuperClass( parser.nextText() );
                     }
-                    else if ( parser.getName().equals( "packageName" ) )
+                    else if ( "packageName".equals( parser.getName() ) )
                     {
                         modelClass.setPackageName( parser.nextText() );
                     }
-                    else if ( parser.getName().equals( "fields" ) )
+                    else if ( "fields".equals( parser.getName() ) )
                     {
                         parseFields( modelClass, parser );
                     }
-                    else if ( parser.getName().equals( "codeSegments" ) )
+                    else if ( "codeSegments".equals( parser.getName() ) )
                     {
                         parseCodeSegment( modelClass, parser );
                     }
@@ -308,7 +320,7 @@ public class ModelReader
     {
         while ( parser.nextTag() == XmlPullParser.START_TAG )
         {
-            if ( parser.getName().equals( "interface" ) )
+            if ( "interface".equals( parser.getName() ) )
             {
                 modelClass.addInterface( parser.nextText() );
             }
@@ -324,48 +336,48 @@ public class ModelReader
     {
         while ( parser.nextTag() == XmlPullParser.START_TAG )
         {
-            if ( parser.getName().equals( "field" ) )
+            if ( "field".equals( parser.getName() ) )
             {
                 ModelField modelField = new ModelField();
 
                 ModelAssociation modelAssociation = null;
 
-                Map fAttributes = getAttributes( parser );
+                Map<String, String> fAttributes = getAttributes( parser );
 
-                Map aAttributes = new HashMap();
+                Map<String, String> aAttributes = new HashMap<String, String>();
 
                 while ( parser.nextTag() == XmlPullParser.START_TAG )
                 {
                     if ( parseBaseElement( modelField, parser ) )
                     {
                     }
-                    else if ( parser.getName().equals( "association" ) )
+                    else if ( "association".equals( parser.getName() ) )
                     {
                         aAttributes = getAttributes( parser );
 
                         modelAssociation = parseAssociation( parser );
                     }
-                    else if ( parser.getName().equals( "alias" ) )
+                    else if ( "alias".equals( parser.getName() ) )
                     {
                         modelField.setAlias( parser.nextText() );
                     }
-                    else if ( parser.getName().equals( "type" ) )
+                    else if ( "type".equals( parser.getName() ) )
                     {
                         modelField.setType( parser.nextText() );
                     }
-                    else if ( parser.getName().equals( "defaultValue" ) )
+                    else if ( "defaultValue".equals( parser.getName() ) )
                     {
                         modelField.setDefaultValue( parser.nextText() );
                     }
-                    else if ( parser.getName().equals( "typeValidator" ) )
+                    else if ( "typeValidator".equals( parser.getName() ) )
                     {
                         modelField.setTypeValidator( parser.nextText() );
                     }
-                    else if ( parser.getName().equals( "required" ) )
+                    else if ( "required".equals( parser.getName() ) )
                     {
                         modelField.setRequired( Boolean.valueOf( parser.nextText() ).booleanValue() );
                     }
-                    else if ( parser.getName().equals( "identifier" ) )
+                    else if ( "identifier".equals( parser.getName() ) )
                     {
                         modelField.setIdentifier( Boolean.valueOf( parser.nextText() ).booleanValue() );
                     }
@@ -393,6 +405,8 @@ public class ModelReader
 
                     modelAssociation.setComment( modelField.getComment() );
 
+                    modelAssociation.setAnnotations( modelField.getAnnotations() );
+
                     // model field fields
                     modelAssociation.setType( modelField.getType() );
 
@@ -436,11 +450,11 @@ public class ModelReader
             if ( parseBaseElement( modelAssociation, parser ) )
             {
             }
-            else if ( parser.getName().equals( "type" ) )
+            else if ( "type".equals( parser.getName() ) )
             {
                 modelAssociation.setTo( parser.nextText() );
             }
-            else if ( parser.getName().equals( "multiplicity" ) )
+            else if ( "multiplicity".equals( parser.getName() ) )
             {
                 modelAssociation.setMultiplicity( parser.nextText() );
             }
@@ -458,7 +472,7 @@ public class ModelReader
     {
         while ( parser.nextTag() == XmlPullParser.START_TAG )
         {
-            if ( parser.getName().equals( "codeSegment" ) )
+            if ( "codeSegment".equals( parser.getName() ) )
             {
                 CodeSegment codeSegment = new CodeSegment();
 
@@ -467,7 +481,7 @@ public class ModelReader
                     if ( parseBaseElement( codeSegment, parser ) )
                     {
                     }
-                    else if ( parser.getName().equals( "code" ) )
+                    else if ( "code".equals( parser.getName() ) )
                     {
                         codeSegment.setCode( parser.nextText() );
                     }
@@ -491,7 +505,7 @@ public class ModelReader
     {
         while ( parser.nextTag() == XmlPullParser.START_TAG )
         {
-            if ( parser.getName().equals( "codeSegment" ) )
+            if ( "codeSegment".equals( parser.getName() ) )
             {
                 CodeSegment codeSegment = new CodeSegment();
 
@@ -500,7 +514,7 @@ public class ModelReader
                     if ( parseBaseElement( codeSegment, parser ) )
                     {
                     }
-                    else if ( parser.getName().equals( "code" ) )
+                    else if ( "code".equals( parser.getName() ) )
                     {
                         codeSegment.setCode( parser.nextText() );
                     }
@@ -522,22 +536,34 @@ public class ModelReader
     private boolean parseBaseElement( BaseElement element, XmlPullParser parser )
         throws XmlPullParserException, IOException
     {
-        if ( parser.getName().equals( "name" ) )
+        if ( "name".equals( parser.getName() ) )
         {
             element.setName( parser.nextText() );
         }
-        else if ( parser.getName().equals( "description" ) )
+        else if ( "description".equals( parser.getName() ) )
         {
             element.setDescription( parser.nextText() );
         }
-        else if ( parser.getName().equals( "version" ) )
+        else if ( "version".equals( parser.getName() ) )
         {
             element.setVersionRange( new VersionRange( parser.nextText() ) );
         }
-        else if ( parser.getName().equals( "comment" ) )
+        else if ( "comment".equals( parser.getName() ) )
         {
             element.setComment( parser.nextText() );
         }
+        else if ( "annotations".equals( parser.getName() ) )
+        {
+            List<String> annotationsList = new ArrayList<String>();
+            while ( parser.nextTag() == XmlPullParser.START_TAG )
+            {
+                if ( "annotation".equals( parser.getName() ) )
+                {
+                    annotationsList.add( parser.nextText() );
+                }
+            }
+            element.setAnnotations( annotationsList );
+        }
         else
         {
             return false;
@@ -546,9 +572,9 @@ public class ModelReader
         return true;
     }
 
-    private Map/*<String, String>*/ getAttributes( XmlPullParser parser )
+    private Map<String, String> getAttributes( XmlPullParser parser )
     {
-        Map attributes = new HashMap();
+        Map<String, String> attributes = new HashMap<String, String>();
 
         for ( int i = 0; i < parser.getAttributeCount(); i++ )
         {
