@@ -34,14 +34,12 @@ import org.codehaus.modello.plugin.java.javasource.JParameter;
 import org.codehaus.modello.plugin.java.javasource.JSourceCode;
 import org.codehaus.modello.plugin.java.javasource.JSourceWriter;
 import org.codehaus.modello.plugin.java.javasource.JType;
-import org.codehaus.modello.plugin.java.metadata.JavaClassMetadata;
 import org.codehaus.modello.plugins.xml.AbstractXmlJavaGenerator;
 import org.codehaus.modello.plugins.xml.metadata.XmlAssociationMetadata;
 import org.codehaus.modello.plugins.xml.metadata.XmlFieldMetadata;
 import org.codehaus.plexus.util.StringUtils;
 
 import java.io.IOException;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Properties;
 
@@ -54,11 +52,16 @@ import java.util.Properties;
 public class Dom4jReaderGenerator
     extends AbstractXmlJavaGenerator
 {
+
+    private boolean requiresDomSupport;
+
     public void generate( Model model, Properties parameters )
         throws ModelloException
     {
         initialize( model, parameters );
 
+        requiresDomSupport = false;
+
         try
         {
             generateDom4jReader();
@@ -85,6 +88,7 @@ public class Dom4jReaderGenerator
         initHeader( jClass );
         suppressAllWarnings( objectModel, jClass );
 
+        jClass.addImport( "java.io.InputStream" );
         jClass.addImport( "java.io.IOException" );
         jClass.addImport( "java.io.Reader" );
         jClass.addImport( "java.net.URL" );
@@ -93,7 +97,6 @@ public class Dom4jReaderGenerator
         jClass.addImport( "java.text.DateFormat" );
         jClass.addImport( "java.text.ParsePosition" );
         jClass.addImport( "java.util.Iterator" );
-        jClass.addImport( "org.codehaus.plexus.util.xml.Xpp3Dom" );
         jClass.addImport( "org.dom4j.Attribute" );
         jClass.addImport( "org.dom4j.Document" );
         jClass.addImport( "org.dom4j.DocumentException" );
@@ -103,15 +106,17 @@ public class Dom4jReaderGenerator
 
         addModelImports( jClass, null );
 
-        // ----------------------------------------------------------------------
-        // Write the parse method which will do the unmarshalling.
-        // ----------------------------------------------------------------------
-
         ModelClass root = objectModel.getClass( objectModel.getRoot( getGeneratedVersion() ), getGeneratedVersion() );
+        JClass rootType = new JClass( root.getName() );
 
-        JMethod unmarshall = new JMethod( "read", new JClass( root.getName() ), null );
+        // ----------------------------------------------------------------------
+        // Write the read(XMLStreamReader,boolean) method which will do the unmarshalling.
+        // ----------------------------------------------------------------------
 
-        unmarshall.addParameter( new JParameter( new JClass( "Reader" ), "reader" ) );
+        JMethod unmarshall = new JMethod( "read", rootType, null );
+        unmarshall.getModifiers().makePrivate();
+
+        unmarshall.addParameter( new JParameter( new JClass( "Document" ), "document" ) );
         unmarshall.addParameter( new JParameter( JType.BOOLEAN, "strict" ) );
 
         unmarshall.addException( new JClass( "IOException" ) );
@@ -119,20 +124,45 @@ public class Dom4jReaderGenerator
 
         JSourceCode sc = unmarshall.getSourceCode();
 
+        String className = root.getName();
+        String variableName = uncapitalise( className );
+
+        sc.add( "String encoding = document.getXMLEncoding();" );
+
+        sc.add( className + ' ' + variableName + " = parse" + root.getName() + "( \"" + resolveTagName( root )
+                + "\", document.getRootElement(), strict );" );
+
+        sc.add( variableName + ".setModelEncoding( encoding );" );
+
+        sc.add( "return " + variableName + ";" );
+
+        jClass.addMethod( unmarshall );
+
+        // ----------------------------------------------------------------------
+        // Write the read(Reader[,boolean]) methods which will do the unmarshalling.
+        // ----------------------------------------------------------------------
+
+        unmarshall = new JMethod( "read", rootType, null );
+
+        unmarshall.addParameter( new JParameter( new JClass( "Reader" ), "reader" ) );
+        unmarshall.addParameter( new JParameter( JType.BOOLEAN, "strict" ) );
+
+        unmarshall.addException( new JClass( "IOException" ) );
+        unmarshall.addException( new JClass( "DocumentException" ) );
+
+        sc = unmarshall.getSourceCode();
+
         sc.add( "SAXReader parser = new SAXReader();" );
 
         sc.add( "Document document = parser.read( reader );" );
 
-        sc.add( "String encoding = document.getXMLEncoding();" );
-
-        sc.add( "return parse" + root.getName() + "( \"" + resolveTagName( root )
-            + "\", document.getRootElement(), strict, encoding );" );
+        sc.add( "return read( document, strict );" );
 
         jClass.addMethod( unmarshall );
 
         // ----------------------------------------------------------------------
 
-        unmarshall = new JMethod( "read", new JClass( root.getName() ), null );
+        unmarshall = new JMethod( "read", rootType, null );
 
         unmarshall.addParameter( new JParameter( new JClass( "Reader" ), "reader" ) );
 
@@ -145,9 +175,48 @@ public class Dom4jReaderGenerator
 
         jClass.addMethod( unmarshall );
 
+        // ----------------------------------------------------------------------
+        // Write the read(InputStream[,boolean]) methods which will do the unmarshalling.
         // ----------------------------------------------------------------------
 
-        unmarshall = new JMethod( "read", new JClass( root.getName() ), null );
+        unmarshall = new JMethod( "read", rootType, null );
+
+        unmarshall.addParameter( new JParameter( new JClass( "InputStream" ), "stream" ) );
+        unmarshall.addParameter( new JParameter( JType.BOOLEAN, "strict" ) );
+
+        unmarshall.addException( new JClass( "IOException" ) );
+        unmarshall.addException( new JClass( "DocumentException" ) );
+
+        sc = unmarshall.getSourceCode();
+
+        sc.add( "SAXReader parser = new SAXReader();" );
+
+        sc.add( "Document document = parser.read( stream );" );
+
+        sc.add( "return read( document, strict );" );
+
+        jClass.addMethod( unmarshall );
+
+        // ----------------------------------------------------------------------
+
+        unmarshall = new JMethod( "read", rootType, null );
+
+        unmarshall.addParameter( new JParameter( new JClass( "InputStream" ), "stream" ) );
+
+        unmarshall.addException( new JClass( "IOException" ) );
+        unmarshall.addException( new JClass( "DocumentException" ) );
+
+        sc = unmarshall.getSourceCode();
+
+        sc.add( "return read( stream, true );" );
+
+        jClass.addMethod( unmarshall );
+
+        // ----------------------------------------------------------------------
+        // Write the read(URL[,boolean]) methods which will do the unmarshalling.
+        // ----------------------------------------------------------------------
+
+        unmarshall = new JMethod( "read", rootType, null );
 
         unmarshall.addParameter( new JParameter( new JClass( "URL" ), "url" ) );
         unmarshall.addParameter( new JParameter( JType.BOOLEAN, "strict" ) );
@@ -161,16 +230,13 @@ public class Dom4jReaderGenerator
 
         sc.add( "Document document = parser.read( url );" );
 
-        sc.add( "String encoding = document.getXMLEncoding();" );
-
-        sc.add( "return parse" + root.getName() + "( \"" + resolveTagName( root )
-            + "\", document.getRootElement(), strict, encoding );" );
+        sc.add( "return read( document, strict );" );
 
         jClass.addMethod( unmarshall );
 
         // ----------------------------------------------------------------------
 
-        unmarshall = new JMethod( "read", new JClass( root.getName() ), null );
+        unmarshall = new JMethod( "read", rootType, null );
 
         unmarshall.addParameter( new JParameter( new JClass( "URL" ), "url" ) );
 
@@ -195,6 +261,12 @@ public class Dom4jReaderGenerator
 
         writeHelpers( jClass );
 
+        if ( requiresDomSupport )
+        {
+            jClass.addImport( "org.codehaus.plexus.util.xml.Xpp3Dom" );
+            writeDomHelpers( jClass );
+        }
+
         // ----------------------------------------------------------------------
         //
         // ----------------------------------------------------------------------
@@ -208,18 +280,8 @@ public class Dom4jReaderGenerator
     {
         ModelClass root = objectModel.getClass( objectModel.getRoot( getGeneratedVersion() ), getGeneratedVersion() );
 
-        for ( Iterator i = objectModel.getClasses( getGeneratedVersion() ).iterator(); i.hasNext(); )
+        for ( ModelClass clazz : getClasses( objectModel ) )
         {
-            ModelClass clazz = (ModelClass) i.next();
-
-            JavaClassMetadata javaClassMetadata = (JavaClassMetadata) clazz.getMetadata( JavaClassMetadata.ID );
-
-            if ( !javaClassMetadata.isEnabled() )
-            {
-                // Skip import of those classes that are not enabled for the java plugin.
-                continue;
-            }
-
             writeClassParser( clazz, jClass, root.getName().equals( clazz.getName() ) );
         }
     }
@@ -238,7 +300,6 @@ public class Dom4jReaderGenerator
         unmarshall.addParameter( new JParameter( new JClass( "String" ), "tagName" ) );
         unmarshall.addParameter( new JParameter( new JClass( "Element" ), "element" ) );
         unmarshall.addParameter( new JParameter( JType.BOOLEAN, "strict" ) );
-        unmarshall.addParameter( new JParameter( new JClass( "String" ), "encoding" ) );
 
         unmarshall.addException( new JClass( "IOException" ) );
         unmarshall.addException( new JClass( "DocumentException" ) );
@@ -249,15 +310,11 @@ public class Dom4jReaderGenerator
 
         ModelField contentField = null;
 
-        List modelFields = modelClass.getAllFields( getGeneratedVersion(), true );
-
-        modelFields = getNonTransientFields( modelFields );
+        List<ModelField> modelFields = getFieldsForXml( modelClass, getGeneratedVersion() );
 
         // read all XML attributes first
-        for ( Iterator i = modelFields.iterator(); i.hasNext(); )
+        for ( ModelField field : modelFields )
         {
-            ModelField field = (ModelField) i.next();
-
             XmlFieldMetadata xmlFieldMetadata = (XmlFieldMetadata) field.getMetadata( XmlFieldMetadata.ID );
 
             if ( xmlFieldMetadata.isAttribute() )
@@ -287,9 +344,6 @@ public class Dom4jReaderGenerator
 
         if ( rootElement )
         {
-            // encoding parameter is only used in root class
-            sc.add( uncapClassName + ".setModelEncoding( encoding );" );
-
             sc.add( "if ( strict )" );
             sc.add( "{" );
             sc.indent();
@@ -319,15 +373,8 @@ public class Dom4jReaderGenerator
 
             sc.add( "Node node = (Node) i.next();" );
 
-            sc.add( "if ( node.getNodeType() != Node.ELEMENT_NODE )" );
-            sc.add( "{" );
-            sc.indent();
-
-            // TODO: attach to model in some way
-
-            sc.unindent();
-            sc.add( "}" );
-            sc.add( "else" );
+            sc.add( "if ( node.getNodeType() == Node.ELEMENT_NODE )" );
+            // TODO: attach other NodeTypes to model in some way
             sc.add( "{" );
             sc.indent();
 
@@ -335,10 +382,8 @@ public class Dom4jReaderGenerator
 
             boolean addElse = false;
 
-            for ( Iterator i = modelFields.iterator(); i.hasNext(); )
+            for ( ModelField field : modelFields )
             {
-                ModelField field = (ModelField) i.next();
-
                 XmlFieldMetadata xmlFieldMetadata = (XmlFieldMetadata) field.getMetadata( XmlFieldMetadata.ID );
 
                 if ( !xmlFieldMetadata.isAttribute() )
@@ -357,11 +402,7 @@ public class Dom4jReaderGenerator
                 sc.indent();
             }
 
-            sc.add( "if ( strict )" );
-
-            sc.add( "{" );
-            sc.addIndented( "throw new DocumentException( \"Unrecognised tag: '\" + childElement.getName() + \"'\" );" );
-            sc.add( "}" );
+            sc.add( "checkUnknownElement( childElement, strict );" );
 
             if ( addElse )
             {
@@ -425,7 +466,7 @@ public class Dom4jReaderGenerator
 
                 sc.add( "{" );
                 sc.addIndented( objectName + ".set" + capFieldName + "( parse" + association.getTo() + "( \""
-                                + fieldTagName + "\", childElement, strict, encoding ) );" );
+                                + fieldTagName + "\", childElement, strict ) );" );
                 sc.add( "}" );
             }
             else
@@ -461,24 +502,15 @@ public class Dom4jReaderGenerator
 
                         sc.add( "Node n = (Node) j.next();" );
 
-                        sc.add( "if ( n.getNodeType() != Node.ELEMENT_NODE )" );
-
-                        sc.add( "{" );
-                        sc.indent();
-
-                        // TODO: track the whitespace in the model
-
-                        sc.unindent();
-                        sc.add( "}" );
-
-                        sc.add( "else" );
+                        sc.add( "if ( n.getNodeType() == Node.ELEMENT_NODE )" );
+                        // TODO: track the whitespace in the model (other NodeTypes)
 
                         sc.add( "{" );
                         sc.indent();
 
                         sc.add( "Element listElement = (Element) n;" );
 
-                        sc.add( "if ( listElement.getName().equals( \"" + valuesTagName + "\" ) )" );
+                        sc.add( "if ( \"" + valuesTagName + "\".equals( listElement.getName() ) )" );
 
                         sc.add( "{" );
                         sc.indent();
@@ -486,7 +518,7 @@ public class Dom4jReaderGenerator
                     else
                     {
                         sc.add( ( addElse ? "else " : "" )
-                            + "if ( childElement.getName().equals( \"" + valuesTagName + "\" ) )" );
+                            + "if ( \"" + valuesTagName + "\".equals( childElement.getName() ) )" );
 
                         sc.add( "{" );
                         sc.indent();
@@ -511,7 +543,7 @@ public class Dom4jReaderGenerator
                     if ( isClassInModel( association.getTo(), field.getModelClass().getModel() ) )
                     {
                         sc.add( associationName + ".add( parse" + association.getTo() + "( \"" + valuesTagName
-                            + "\", listElement, strict, encoding ) );" );
+                            + "\", listElement, strict ) );" );
                     }
                     else
                     {
@@ -562,24 +594,15 @@ public class Dom4jReaderGenerator
 
                         sc.add( "Node n = (Node) j.next();" );
 
-                        sc.add( "if ( n.getNodeType() != Node.ELEMENT_NODE )" );
-
-                        sc.add( "{" );
-                        sc.indent();
-
-                        // TODO: track the whitespace in the model
-
-                        sc.unindent();
-                        sc.add( "}" );
-
-                        sc.add( "else" );
+                        sc.add( "if ( n.getNodeType() == Node.ELEMENT_NODE )" );
+                        // TODO: track the whitespace in the model (other NodeTypes)
 
                         sc.add( "{" );
                         sc.indent();
 
                         sc.add( "Element listElement = (Element) n;" );
 
-                        sc.add( "if ( listElement.getName().equals( \"" + valuesTagName + "\" ) )" );
+                        sc.add( "if ( \"" + valuesTagName + "\".equals( listElement.getName() ) )" );
 
                         sc.add( "{" );
                         sc.indent();
@@ -597,30 +620,21 @@ public class Dom4jReaderGenerator
 
                         sc.add( "Node nd = (Node) k.next();" );
 
-                        sc.add( "if ( nd.getNodeType() != Node.ELEMENT_NODE )" );
-
-                        sc.add( "{" );
-                        sc.indent();
-
-                        // TODO: track the whitespace in the model
-
-                        sc.unindent();
-                        sc.add( "}" );
-
-                        sc.add( "else" );
+                        sc.add( "if ( nd.getNodeType() == Node.ELEMENT_NODE )" );
+                        // TODO: track the whitespace in the model (other NodeTypes)
 
                         sc.add( "{" );
                         sc.indent();
 
                         sc.add( "Element propertyElement = (Element) nd;" );
 
-                        sc.add( "if ( propertyElement.getName().equals( \"key\" ) )" );
+                        sc.add( "if ( \"key\".equals( propertyElement.getName() ) )" );
 
                         sc.add( "{" );
                         sc.addIndented( "key = propertyElement.getText();" );
                         sc.add( "}" );
 
-                        sc.add( "else if ( propertyElement.getName().equals( \"value\" ) )" );
+                        sc.add( "else if ( \"value\".equals( propertyElement.getName() ) )" );
 
                         sc.add( "{" );
                         sc.addIndented( "value = propertyElement.getText()"
@@ -660,17 +674,8 @@ public class Dom4jReaderGenerator
 
                         sc.add( "Node n = (Node) j.next();" );
 
-                        sc.add( "if ( n.getNodeType() != Node.ELEMENT_NODE )" );
-
-                        sc.add( "{" );
-                        sc.indent();
-
-                        // TODO: track the whitespace in the model
-
-                        sc.unindent();
-                        sc.add( "}" );
-
-                        sc.add( "else" );
+                        sc.add( "if ( n.getNodeType() == Node.ELEMENT_NODE )" );
+                        // TODO: track the whitespace in the model (other NodeTypes)
 
                         sc.add( "{" );
                         sc.indent();
@@ -796,6 +801,8 @@ public class Dom4jReaderGenerator
         else if ( "DOM".equals( type ) )
         {
             sc.add( objectName + "." + setterName + "( writeElementToXpp3Dom( " + childElementName + " ) );" );
+
+            requiresDomSupport = true;
         }
         else
         {
@@ -944,18 +951,67 @@ public class Dom4jReaderGenerator
         method.addParameter( new JParameter( new JClass( "String" ), "dateFormat" ) );
         method.addException( new JClass( "DocumentException" ) );
 
-        writeDateParsingHelper( method.getSourceCode(), "new DocumentException( e.getMessage() )" );
+        writeDateParsingHelper( method.getSourceCode(), "new DocumentException( e.getMessage(), e )" );
 
         jClass.addMethod( method );
 
         // --------------------------------------------------------------------
 
-        method = new JMethod( "writeElementToXpp3Dom", new JClass( "Xpp3Dom" ), null );
+        method = new JMethod( "checkFieldWithDuplicate", JType.BOOLEAN, null );
+        method.getModifiers().makePrivate();
+
+        method.addParameter( new JParameter( new JClass( "Element" ), "element" ) );
+        method.addParameter( new JParameter( new JClass( "String" ), "tagName" ) );
+        method.addParameter( new JParameter( new JClass( "String" ), "alias" ) );
+        method.addParameter( new JParameter( new JClass( "java.util.Set" ), "parsed" ) );
+        method.addException( new JClass( "DocumentException" ) );
+
+        sc = method.getSourceCode();
+
+        sc.add( "if ( !( element.getName().equals( tagName ) || element.getName().equals( alias ) ) )" );
+
+        sc.add( "{" );
+        sc.addIndented( "return false;" );
+        sc.add( "}" );
+
+        sc.add( "if ( !parsed.add( tagName ) )" );
+
+        sc.add( "{" );
+        sc.addIndented( "throw new DocumentException( \"Duplicated tag: '\" + tagName + \"'\" );" );
+        sc.add( "}" );
+
+        sc.add( "return true;" );
+
+        jClass.addMethod( method );
+
+        // --------------------------------------------------------------------
+
+        method = new JMethod( "checkUnknownElement", null, null );
+        method.getModifiers().makePrivate();
+
+        method.addParameter( new JParameter( new JClass( "Element" ), "element" ) );
+        method.addParameter( new JParameter( JType.BOOLEAN, "strict" ) );
+        method.addException( new JClass( "DocumentException" ) );
+
+        sc = method.getSourceCode();
+
+        sc.add( "if ( strict )" );
+
+        sc.add( "{" );
+        sc.addIndented( "throw new DocumentException( \"Unrecognised tag: '\" + element.getName() + \"'\" );" );
+        sc.add( "}" );
+
+        jClass.addMethod( method );
+    }
+
+    private void writeDomHelpers( JClass jClass )
+    {
+        JMethod method = new JMethod( "writeElementToXpp3Dom", new JClass( "Xpp3Dom" ), null );
         method.getModifiers().makePrivate();
 
         method.addParameter( new JParameter( new JClass( "Element" ), "element" ) );
 
-        sc = method.getSourceCode();
+        JSourceCode sc = method.getSourceCode();
 
         sc.add( "Xpp3Dom xpp3Dom = new Xpp3Dom( element.getName() );" );
 
@@ -989,38 +1045,6 @@ public class Dom4jReaderGenerator
         sc.add( "return xpp3Dom;" );
 
         jClass.addMethod( method );
-
-        // --------------------------------------------------------------------
-
-        method = new JMethod( "checkFieldWithDuplicate", JType.BOOLEAN, null );
-        method.getModifiers().makePrivate();
-
-        method.addParameter( new JParameter( new JClass( "Element" ), "element" ) );
-        method.addParameter( new JParameter( new JClass( "String" ), "tagName" ) );
-        method.addParameter( new JParameter( new JClass( "String" ), "alias" ) );
-        method.addParameter( new JParameter( new JClass( "java.util.Set" ), "parsed" ) );
-        method.addException( new JClass( "DocumentException" ) );
-
-        sc = method.getSourceCode();
-
-        sc.add( "if ( !( element.getName().equals( tagName ) || element.getName().equals( alias ) ) )" );
-
-        sc.add( "{" );
-        sc.addIndented( "return false;" );
-        sc.add( "}" );
-
-        sc.add( "if ( parsed.contains( tagName ) )" );
-
-        sc.add( "{" );
-        sc.addIndented(
-            "throw new DocumentException( \"Duplicated tag: '\" + tagName + \"'\" );" );
-        sc.add( "}" );
-
-        sc.add( "parsed.add( tagName );" );
-
-        sc.add( "return true;" );
-
-        jClass.addMethod( method );
     }
 
     private JMethod convertNumericalType( String methodName, JType returnType, String expression, String typeDesc )
@@ -1046,7 +1070,7 @@ public class Dom4jReaderGenerator
         sc.addIndented( "return " + expression + ";" );
         sc.add( "}" );
 
-        sc.add( "catch ( NumberFormatException e )" );
+        sc.add( "catch ( NumberFormatException nfe )" );
 
         sc.add( "{" );
         sc.indent();
@@ -1055,7 +1079,7 @@ public class Dom4jReaderGenerator
 
         sc.add( "{" );
         sc.addIndented( "throw new DocumentException( \"Unable to parse element '\" + attribute + \"', must be "
-                        + typeDesc + "\" );" );
+                        + typeDesc + "\", nfe );" );
         sc.add( "}" );
 
         sc.unindent();
