@@ -33,7 +33,6 @@ import org.codehaus.modello.plugin.java.javasource.JMethod;
 import org.codehaus.modello.plugin.java.javasource.JParameter;
 import org.codehaus.modello.plugin.java.javasource.JSourceCode;
 import org.codehaus.modello.plugin.java.javasource.JSourceWriter;
-import org.codehaus.modello.plugin.java.metadata.JavaClassMetadata;
 import org.codehaus.modello.plugin.java.metadata.JavaFieldMetadata;
 import org.codehaus.modello.plugin.model.ModelClassMetadata;
 import org.codehaus.modello.plugins.xml.AbstractXmlJavaGenerator;
@@ -42,7 +41,6 @@ import org.codehaus.modello.plugins.xml.metadata.XmlFieldMetadata;
 import org.codehaus.modello.plugins.xml.metadata.XmlModelMetadata;
 
 import java.io.IOException;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Properties;
 
@@ -56,11 +54,16 @@ import java.util.Properties;
 public class Dom4jWriterGenerator
     extends AbstractXmlJavaGenerator
 {
+
+    private boolean requiresDomSupport;
+
     public void generate( Model model, Properties parameters )
         throws ModelloException
     {
         initialize( model, parameters );
 
+        requiresDomSupport = false;
+
         try
         {
             generateDom4jWriter();
@@ -87,12 +90,12 @@ public class Dom4jWriterGenerator
         initHeader( jClass );
         suppressAllWarnings( objectModel, jClass );
 
+        jClass.addImport( "java.io.OutputStream" );
         jClass.addImport( "java.io.Writer" );
         jClass.addImport( "java.util.Arrays" );
         jClass.addImport( "java.util.Iterator" );
         jClass.addImport( "java.util.Locale" );
         jClass.addImport( "java.text.DateFormat" );
-        jClass.addImport( "org.codehaus.plexus.util.xml.Xpp3Dom" );
         jClass.addImport( "org.dom4j.Document" );
         jClass.addImport( "org.dom4j.DocumentException" );
         jClass.addImport( "org.dom4j.DocumentFactory" );
@@ -107,13 +110,16 @@ public class Dom4jWriterGenerator
         ModelClass rootClass = objectModel.getClass( root, getGeneratedVersion() );
 
         String rootElement = resolveTagName( rootClass );
+        String variableName = uncapitalise( root );
 
-        // Write the parse method which will do the unmarshalling.
+        // ----------------------------------------------------------------------
+        // Write the write( Reader, Model ) method which will do the unmarshalling.
+        // ----------------------------------------------------------------------
 
         JMethod marshall = new JMethod( "write" );
 
         marshall.addParameter( new JParameter( new JClass( "Writer" ), "writer" ) );
-        marshall.addParameter( new JParameter( new JClass( root ), rootElement ) );
+        marshall.addParameter( new JParameter( new JClass( root ), variableName ) );
 
         marshall.addException( new JClass( "java.io.IOException" ) );
 
@@ -121,7 +127,7 @@ public class Dom4jWriterGenerator
 
         sc.add( "Document document = new DocumentFactory().createDocument();" );
 
-        sc.add( "write" + root + "( " + rootElement + ", \"" + rootElement + "\", document );" );
+        sc.add( "write" + root + "( " + variableName + ", \"" + rootElement + "\", document );" );
 
         // TODO: pretty printing optional
         sc.add( "OutputFormat format = OutputFormat.createPrettyPrint();" );
@@ -132,9 +138,40 @@ public class Dom4jWriterGenerator
 
         jClass.addMethod( marshall );
 
+        // ----------------------------------------------------------------------
+        // Write the write( OutputStream, Model ) method which will do the unmarshalling.
+        // ----------------------------------------------------------------------
+
+        marshall = new JMethod( "write" );
+
+        marshall.addParameter( new JParameter( new JClass( "OutputStream" ), "stream" ) );
+        marshall.addParameter( new JParameter( new JClass( root ), variableName ) );
+
+        marshall.addException( new JClass( "java.io.IOException" ) );
+
+        sc = marshall.getSourceCode();
+
+        sc.add( "Document document = new DocumentFactory().createDocument();" );
+
+        sc.add( "write" + root + "( " + variableName + ", \"" + rootElement + "\", document );" );
+
+        // TODO: pretty printing optional
+        sc.add( "OutputFormat format = OutputFormat.createPrettyPrint();" );
+        sc.add( "format.setLineSeparator( System.getProperty( \"line.separator\" ) );" );
+        sc.add( "format.setEncoding( " + variableName + ".getModelEncoding() );" );
+        sc.add( "XMLWriter serializer = new XMLWriter( stream, format );" );
+
+        sc.add( "serializer.write( document );" );
+
+        jClass.addMethod( marshall );
+
         writeAllClasses( objectModel, jClass );
 
-        writeHelpers( jClass );
+        if ( requiresDomSupport )
+        {
+            jClass.addImport( "org.codehaus.plexus.util.xml.Xpp3Dom" );
+            writeDomHelpers( jClass );
+        }
 
         jClass.print( sourceWriter );
 
@@ -144,18 +181,8 @@ public class Dom4jWriterGenerator
     private void writeAllClasses( Model objectModel, JClass jClass )
         throws ModelloException
     {
-        for ( Iterator i = objectModel.getClasses( getGeneratedVersion() ).iterator(); i.hasNext(); )
+        for ( ModelClass clazz : getClasses( objectModel ) )
         {
-            ModelClass clazz = (ModelClass) i.next();
-
-            JavaClassMetadata javaClassMetadata = (JavaClassMetadata) clazz.getMetadata( JavaClassMetadata.ID );
-
-            if ( !javaClassMetadata.isEnabled() )
-            {
-                // Skip import of those classes that are not enabled for the java plugin.
-                continue;
-            }
-
             writeClass( clazz, jClass );
         }
     }
@@ -175,14 +202,7 @@ public class Dom4jWriterGenerator
 
         ModelClassMetadata classMetadata = (ModelClassMetadata) modelClass.getMetadata( ModelClassMetadata.ID );
 
-        if ( classMetadata.isRootElement() )
-        {
-            marshall.addParameter( new JParameter( new JClass( "Document" ), "parentElement" ) );
-        }
-        else
-        {
-            marshall.addParameter( new JParameter( new JClass( "Element" ), "parentElement" ) );
-        }
+        marshall.addParameter( new JParameter( new JClass( "org.dom4j.Branch" ), "parentElement" ) );
 
         marshall.addException( new JClass( "java.io.IOException" ) );
 
@@ -219,15 +239,11 @@ public class Dom4jWriterGenerator
 
         String contentValue = null;
 
-        List modelFields = modelClass.getAllFields( getGeneratedVersion(), true );
-
-        modelFields = getNonTransientFields( modelFields );
+        List<ModelField> modelFields = getFieldsForXml( modelClass, getGeneratedVersion() );
 
         // XML attributes
-        for ( Iterator i = modelFields.iterator(); i.hasNext(); )
+        for ( ModelField field : modelFields )
         {
-            ModelField field = (ModelField) i.next();
-
             XmlFieldMetadata xmlFieldMetadata = (XmlFieldMetadata) field.getMetadata( XmlFieldMetadata.ID );
 
             JavaFieldMetadata javaFieldMetadata = (JavaFieldMetadata) field.getMetadata( JavaFieldMetadata.ID );
@@ -263,10 +279,8 @@ public class Dom4jWriterGenerator
         }
 
         // XML tags
-        for ( Iterator fieldIterator = modelFields.iterator(); fieldIterator.hasNext(); )
+        for ( ModelField field : modelFields )
         {
-            ModelField field = (ModelField) fieldIterator.next();
-
             XmlFieldMetadata xmlFieldMetadata = (XmlFieldMetadata) field.getMetadata( XmlFieldMetadata.ID );
 
             if ( xmlFieldMetadata.isContent() )
@@ -422,6 +436,8 @@ public class Dom4jWriterGenerator
             if ( "DOM".equals( field.getType() ) )
             {
                 sc.add( "writeXpp3DomToElement( (Xpp3Dom) " + value + ", element );" );
+
+                requiresDomSupport = true;
             }
             else
             {
@@ -434,7 +450,7 @@ public class Dom4jWriterGenerator
         }
     }
 
-    private void writeHelpers( JClass jClass )
+    private void writeDomHelpers( JClass jClass )
     {
         JMethod method = new JMethod( "writeXpp3DomToElement" );
         method.getModifiers().makePrivate();
