@@ -40,7 +40,7 @@
  *
  * Copyright 2001-2002 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: JInterface.java 1297 2009-07-23 01:17:35Z hboutemy $
+ * $Id: JInterface.java 1413 2010-02-13 21:23:01Z hboutemy $
  */
 
 package org.codehaus.modello.plugin.java.javasource;
@@ -69,7 +69,6 @@ package org.codehaus.modello.plugin.java.javasource;
 
 import java.util.ArrayList;
 import java.util.Enumeration;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Vector;
 
@@ -81,7 +80,7 @@ import java.util.Vector;
  *
  * @author <a href="mailto:skopp@riege.de">Martin Skopp</a>
  * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
- * @version $Revision: 1297 $ $Date: 2009-07-23 03:17:35 +0200 (Do, 23. Jul 2009) $
+ * @version $Revision: 1413 $ $Date: 2010-02-13 22:23:01 +0100 (sam. 13 févr. 2010) $
  **/
 public final class JInterface extends JStructure
 {
@@ -95,7 +94,7 @@ public final class JInterface extends JStructure
     /**
      * The list of methods of this JInterface
      */
-    private Vector methods = null;
+    private Vector<JMethodSignature> methods = null;
 
 
     /**
@@ -109,7 +108,7 @@ public final class JInterface extends JStructure
         throws IllegalArgumentException
     {
         super( name );
-        methods = new Vector();
+        methods = new Vector<JMethodSignature>();
 
         //-- initialize default Java doc
         getJDocComment().appendComment( "Interface " + getLocalName() + "." );
@@ -414,7 +413,7 @@ public final class JInterface extends JStructure
 
         if ( getInterfaceCount() > 0 )
         {
-            Enumeration e = getInterfaces();
+            Enumeration<String> e = getInterfaces();
             buffer.append( "extends " );
             while ( e.hasMoreElements() )
             {
@@ -500,9 +499,9 @@ public final class JInterface extends JStructure
             jsw.writeln( ';' );
         }
 
-        for ( Iterator iterator = sourceCodeEntries.iterator(); iterator.hasNext(); )
+        for ( String sourceCodeEntry : sourceCodeEntries )
         {
-            jsw.writeln( (String) iterator.next() );
+            jsw.writeln( sourceCodeEntry );
         }
 
         jsw.unindent();
@@ -511,7 +510,7 @@ public final class JInterface extends JStructure
         jsw.close();
     } //-- printSource
 
-    private List sourceCodeEntries = new ArrayList();
+    private List<String> sourceCodeEntries = new ArrayList<String>();
 
     public void addSourceCode( String sourceCode )
     {
