@@ -40,7 +40,7 @@
  *
  * Copyright 2001-2002 (C) Intalio, Inc. All Rights Reserved.
  *
- * $Id: JStructure.java 1311 2009-08-20 23:28:35Z hboutemy $
+ * $Id: JStructure.java 1413 2010-02-13 21:23:01Z hboutemy $
  */
 
 package org.codehaus.modello.plugin.java.javasource;
@@ -83,7 +83,7 @@ import org.codehaus.plexus.util.WriterFactory;
  *
  * @author <a href="mailto:skopp@riege.de">Martin Skopp</a>
  * @author <a href="mailto:kvisco@intalio.com">Keith Visco</a>
- * @version $Revision: 1311 $ $Date: 2009-08-21 01:28:35 +0200 (Fr, 21. Aug 2009) $
+ * @version $Revision: 1413 $ $Date: 2010-02-13 22:23:01 +0100 (sam. 13 févr. 2010) $
  */
 public abstract class JStructure extends JType
 {
@@ -111,12 +111,12 @@ public abstract class JStructure extends JType
     /**
      * List of imported classes and packages
      */
-    private Vector imports = null;
+    private Vector<String> imports = null;
 
     /**
      * The set of interfaces implemented/extended by this JStructure
      */
-    private Vector interfaces = null;
+    private Vector<String> interfaces = null;
 
     /**
      * The Javadoc for this JStructure
@@ -162,8 +162,8 @@ public abstract class JStructure extends JType
             throw new IllegalArgumentException( err );
         }
         this.packageName = getPackageFromClassName( name );
-        imports = new Vector();
-        interfaces = new Vector();
+        imports = new Vector<String>();
+        interfaces = new Vector<String>();
         jdc = new JDocComment();
         modifiers = new JModifiers();
         //-- initialize default Java doc
@@ -225,7 +225,7 @@ public abstract class JStructure extends JType
             //-- This may cause issues if the current package
             //-- defines any classes that have the same name
             //-- name as the java.lang package.
-            if ( pkgName.equals( "java.lang" ) ) return;
+            if ( "java.lang".equals( pkgName ) ) return;
 
             //-- for readabilty keep import list sorted, and make sure
             //-- we do not include more than one of the same import
@@ -416,7 +416,7 @@ public abstract class JStructure extends JType
      *
      * @return the Enumeration of imports. May be empty.
      */
-    public Enumeration getImports()
+    public Enumeration<String> getImports()
     {
         return imports.elements();
     } //-- getImports
@@ -428,7 +428,7 @@ public abstract class JStructure extends JType
      * @return the Enumeration of interface names for this
      * JStructure. May be empty.
      */
-    public Enumeration getInterfaces()
+    public Enumeration<String> getInterfaces()
     {
         return interfaces.elements();
     } //-- getInterfaces
@@ -676,7 +676,7 @@ public abstract class JStructure extends JType
             jsw.writeln( " //- Imported classes and packages -/" );
             jsw.writeln( "//---------------------------------/" );
             jsw.writeln();
-            Enumeration e = imports.elements();
+            Enumeration<String> e = imports.elements();
             while ( e.hasMoreElements() )
             {
                 jsw.write( "import " );
