@@ -20,26 +20,26 @@ import java.util.List;
 
 /**
  * Test Interface used by {@link JavaGeneratorTest#testBiDirectionalJavaGenerator()} to ensure that
- * interface overrides can be used. 
+ * interface overrides can be used.
  *
  * @author <a href="mailto:joakim@erdfelt.com">Joakim Erdfelt</a>
- * @version $Id: Role.java 1094 2009-01-02 22:06:17Z hboutemy $
+ * @version $Id: Role.java 1415 2010-02-13 22:23:37Z hboutemy $
  */
 public interface Role
 {
     public String getName();
-    
+
     public void setName(String name);
-    
-    public void setRoles(List roles);
-    
-    public List getRoles();
-    
+
+    public void setRoles(List<Role> roles);
+
+    public List<Role> getRoles();
+
     public void addRole(Role role);
-    
+
     public void removeRole(Role role);
-    
+
     public void setPermission(Permission permission);
-    
+
     public Permission getPermission();
 }
