@@ -32,6 +32,7 @@ import org.codehaus.modello.test.features.BidiInSet;
 import org.codehaus.modello.test.features.InterfacesFeature;
 import org.codehaus.modello.test.features.JavaAbstractFeature;
 import org.codehaus.modello.test.features.JavaFeatures;
+import org.codehaus.modello.test.features.NodeItem;
 import org.codehaus.modello.test.features.Reference;
 import org.codehaus.modello.test.features.SimpleInterface;
 import org.codehaus.modello.test.features.SimpleTypes;
@@ -61,7 +62,7 @@ import junit.framework.Assert;
 
 /**
  * @author Herve Boutemy
- * @version $Id: JavaVerifier.java 1280 2009-07-07 20:59:28Z hboutemy $
+ * @version $Id: JavaVerifier.java 1484 2010-05-08 17:25:54Z bentmann $
  */
 public class JavaVerifier
     extends Verifier
@@ -198,10 +199,17 @@ public class JavaVerifier
         Assert.assertEquals( "primitiveChar"   , 'H'            , simple.getPrimitiveChar() );
         Assert.assertEquals( "primitiveShort"  , (short) 1212   , simple.getPrimitiveShort() );
         Assert.assertEquals( "primitiveInt"    , 121212         , simple.getPrimitiveInt() );
-        Assert.assertEquals( "primitiveLong"   , 12121212       , simple.getPrimitiveLong() );
+        Assert.assertEquals( "primitiveLong"   , 1234567890123L , simple.getPrimitiveLong() );
         Assert.assertEquals( "primitiveFloat"  , 12.12f         , simple.getPrimitiveFloat(), 0f );
         Assert.assertEquals( "primitiveDouble" , 12.12          , simple.getPrimitiveDouble(), 0 );
-        Assert.assertEquals( "objectBoolean"   , Boolean.FALSE  , simple.isObjectBoolean() );
+        Assert.assertEquals( "objectBoolean"   , Boolean.TRUE   , simple.isObjectBoolean() );
+        Assert.assertEquals( "objectByte"      , 12             , simple.getObjectByte().byteValue() );
+        Assert.assertEquals( "objectChar"      , 'H'            , simple.getObjectCharacter().charValue() );
+        Assert.assertEquals( "objectShort"     , (short) 1212   , simple.getObjectShort().shortValue() );
+        Assert.assertEquals( "objectInt"       , 121212         , simple.getObjectInteger().intValue() );
+        Assert.assertEquals( "objectLong"      , 1234567890123L , simple.getObjectLong().longValue() );
+        Assert.assertEquals( "objectFloat"     , 12.12f         , simple.getObjectFloat().floatValue(), 0f );
+        Assert.assertEquals( "objectDouble"    , 12.12          , simple.getObjectDouble().doubleValue(), 0 );
         Assert.assertEquals( "objectString"    , "default value", simple.getObjectString() );
 
         XmlAttributes xmlAttributes = new XmlAttributes();
@@ -210,10 +218,17 @@ public class JavaVerifier
         Assert.assertEquals( "primitiveChar"   , 'H'            , xmlAttributes.getPrimitiveChar() );
         Assert.assertEquals( "primitiveShort"  , (short) 1212   , xmlAttributes.getPrimitiveShort() );
         Assert.assertEquals( "primitiveInt"    , 121212         , xmlAttributes.getPrimitiveInt() );
-        Assert.assertEquals( "primitiveLong"   , 12121212       , xmlAttributes.getPrimitiveLong() );
+        Assert.assertEquals( "primitiveLong"   , 1234567890123L , xmlAttributes.getPrimitiveLong() );
         Assert.assertEquals( "primitiveFloat"  , 12.12f         , xmlAttributes.getPrimitiveFloat(), 0f );
         Assert.assertEquals( "primitiveDouble" , 12.12          , xmlAttributes.getPrimitiveDouble(), 0 );
-        Assert.assertEquals( "objectBoolean"   , Boolean.FALSE  , xmlAttributes.isObjectBoolean() );
+        Assert.assertEquals( "objectBoolean"   , Boolean.TRUE   , xmlAttributes.isObjectBoolean() );
+        Assert.assertEquals( "objectByte"      , 12             , xmlAttributes.getObjectByte().byteValue() );
+        Assert.assertEquals( "objectChar"      , 'H'            , xmlAttributes.getObjectCharacter().charValue() );
+        Assert.assertEquals( "objectShort"     , (short) 1212   , xmlAttributes.getObjectShort().shortValue() );
+        Assert.assertEquals( "objectInt"       , 121212         , xmlAttributes.getObjectInteger().intValue() );
+        Assert.assertEquals( "objectLong"      , 1234567890123L , xmlAttributes.getObjectLong().longValue() );
+        Assert.assertEquals( "objectFloat"     , 12.12f         , xmlAttributes.getObjectFloat().floatValue(), 0f );
+        Assert.assertEquals( "objectDouble"    , 12.12          , xmlAttributes.getObjectDouble().doubleValue(), 0 );
         Assert.assertEquals( "objectString"    , "default value", xmlAttributes.getObjectString() );
     }
 
@@ -312,6 +327,13 @@ public class JavaVerifier
         association.removeSetOfBidi( bidiInSet );
         Assert.assertEquals( 0, association.getSetOfBidis().size() );
 
+        // class with single association to itself, but not bidi!
+        NodeItem parentNode = new NodeItem();
+        NodeItem childNode = new NodeItem();
+        parentNode.setChild( childNode );
+        assertSame( childNode, parentNode.getChild() );
+        assertNull( childNode.getChild() );
+
         // java.useInterface
         checkField( JavaFeatures.class, "useInterface", SubClassLevel1.class, "getUseInterface", "setUseInterface",
                     BaseClass.class);
