@@ -30,14 +30,12 @@ import org.codehaus.modello.plugin.java.javasource.JParameter;
 import org.codehaus.modello.plugin.java.javasource.JSourceCode;
 import org.codehaus.modello.plugin.java.javasource.JSourceWriter;
 import org.codehaus.modello.plugin.java.javasource.JType;
-import org.codehaus.modello.plugin.java.metadata.JavaClassMetadata;
 import org.codehaus.modello.plugin.java.metadata.JavaFieldMetadata;
 import org.codehaus.modello.plugins.xml.metadata.XmlAssociationMetadata;
 import org.codehaus.modello.plugins.xml.metadata.XmlFieldMetadata;
 
 import java.io.IOException;
 import java.util.ArrayList;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Properties;
 
@@ -48,10 +46,15 @@ public class JDOMWriterGenerator
     extends AbstractJDOMGenerator
 {
 
+    private boolean requiresDomSupport;
+
     public void generate( Model model, Properties parameters )
         throws ModelloException
     {
         initialize( model, parameters );
+
+        requiresDomSupport = false;
+
         try
         {
             generateJDOMWriter();
@@ -100,7 +103,6 @@ public class JDOMWriterGenerator
         jClass.addImport( "org.jdom.Text" );
         jClass.addImport( "org.jdom.output.Format" );
         jClass.addImport( "org.jdom.output.XMLOutputter" );
-        jClass.addImport( "org.codehaus.plexus.util.xml.Xpp3Dom" );
 
         addModelImports( jClass, null );
 
@@ -129,7 +131,15 @@ public class JDOMWriterGenerator
         jClass.addMethods( generateUtilityMethods() );
 
         writeAllClasses( objectModel, jClass, rootClass );
+
+        if ( requiresDomSupport )
+        {
+            jClass.addImport( "org.codehaus.plexus.util.xml.Xpp3Dom" );
+            jClass.addMethods( generateDomMethods() );
+        }
+
         jClass.print( sourceWriter );
+
         sourceWriter.close();
     }
 
@@ -166,9 +176,11 @@ public class JDOMWriterGenerator
 
     private JMethod generateWriteModel( String root, String rootElement )
     {
+        String variableName = uncapitalise( root );
+
         JMethod marshall = new JMethod( "write" );
 
-        marshall.addParameter( new JParameter( new JClass( root ), rootElement ) );
+        marshall.addParameter( new JParameter( new JClass( root ), variableName ) );
         marshall.addParameter( new JParameter( new JClass( "Document" ), "document" ) );
         marshall.addParameter( new JParameter( new JClass( "OutputStream" ), "stream" ) );
         marshall.addException( new JClass( "java.io.IOException" ) );
@@ -176,7 +188,7 @@ public class JDOMWriterGenerator
         marshall.getJDocComment().appendComment( "\n@deprecated" );
 
         JSourceCode sc = marshall.getSourceCode();
-        sc.add( "update" + root + "( " + rootElement + ", \"" + rootElement
+        sc.add( "update" + root + "( " + variableName + ", \"" + rootElement
                 + "\", new Counter( 0 ), document.getRootElement() );" );
         sc.add( "XMLOutputter outputter = new XMLOutputter();" );
         sc.add( "outputter.setFormat( Format.getPrettyFormat()" );
@@ -190,9 +202,11 @@ public class JDOMWriterGenerator
 
     private JMethod generateWriteModel2( String root, String rootElement )
     {
+        String variableName = uncapitalise( root );
+
         JMethod marshall = new JMethod( "write" );
 
-        marshall.addParameter( new JParameter( new JClass( root ), rootElement ) );
+        marshall.addParameter( new JParameter( new JClass( root ), variableName ) );
         marshall.addParameter( new JParameter( new JClass( "Document" ), "document" ) );
         marshall.addParameter( new JParameter( new JClass( "OutputStreamWriter" ), "writer" ) );
         marshall.addException( new JClass( "java.io.IOException" ) );
@@ -201,23 +215,25 @@ public class JDOMWriterGenerator
         sc.add( "Format format = Format.getRawFormat()" );
         sc.add( "    .setEncoding( writer.getEncoding() )" );
         sc.add( "    .setLineSeparator( System.getProperty( \"line.separator\" ) );" );
-        sc.add( "write( " + rootElement + ", document, writer, format );" );
+        sc.add( "write( " + variableName + ", document, writer, format );" );
         return marshall;
 
     }
 
     private JMethod generateWriteModel3( String root, String rootElement )
     {
+        String variableName = uncapitalise( root );
+
         JMethod marshall = new JMethod( "write" );
 
-        marshall.addParameter( new JParameter( new JClass( root ), rootElement ) );
+        marshall.addParameter( new JParameter( new JClass( root ), variableName ) );
         marshall.addParameter( new JParameter( new JClass( "Document" ), "document" ) );
         marshall.addParameter( new JParameter( new JClass( "Writer" ), "writer" ) );
         marshall.addParameter( new JParameter( new JClass( "Format" ), "jdomFormat" ) );
         marshall.addException( new JClass( "java.io.IOException" ) );
 
         JSourceCode sc = marshall.getSourceCode();
-        sc.add( "update" + root + "( " + rootElement + ", \"" + rootElement
+        sc.add( "update" + root + "( " + variableName + ", \"" + rootElement
             + "\", new Counter( 0 ), document.getRootElement() );" );
         sc.add( "XMLOutputter outputter = new XMLOutputter();" );
         sc.add( "outputter.setFormat( jdomFormat );" );
@@ -266,19 +282,22 @@ public class JDOMWriterGenerator
         updateElement.getModifiers().makeProtected();
         sc = updateElement.getSourceCode();
         sc.add( "Element element =  parent.getChild( name, parent.getNamespace() );" );
-        sc.add( "if ( element != null && shouldExist )" );
+
+        sc.add( "if ( shouldExist )" );
         sc.add( "{" );
-        sc.addIndented( "counter.increaseCount();" );
-        sc.add( "}" );
-        sc.add( "if ( element == null && shouldExist )" );
+        sc.indent();
+        sc.add( "if ( element == null )" );
         sc.add( "{" );
         sc.indent();
         sc.add( "element = factory.element( name, parent.getNamespace() );" );
         sc.add( "insertAtPreferredLocation( parent, element, counter );" );
+        sc.unindent();
+        sc.add( "}" );
         sc.add( "counter.increaseCount();" );
         sc.unindent();
         sc.add( "}" );
-        sc.add( "if ( !shouldExist && element != null )" );
+
+        sc.add( "else if ( element != null )" );
         sc.add( "{" );
         sc.indent();
         sc.add( "int index = parent.indexOf( element );" );
@@ -479,13 +498,18 @@ public class JDOMWriterGenerator
         sc.add( "}" );
         sc.add( "return element;" );
 
+        return new JMethod[] { findRSElement, updateElement, insAtPref, findRSProps, findRSLists };
+    }
+
+    private JMethod[] generateDomMethods()
+    {
         JMethod findRSDom = new JMethod( "findAndReplaceXpp3DOM", new JClass( "Element" ), null );
         findRSDom.addParameter( new JParameter( new JClass( "Counter" ), "counter" ) );
         findRSDom.addParameter( new JParameter( new JClass( "Element" ), "parent" ) );
         findRSDom.addParameter( new JParameter( new JClass( "String" ), "name" ) );
         findRSDom.addParameter( new JParameter( new JClass( "Xpp3Dom" ), "dom" ) );
         findRSDom.getModifiers().makeProtected();
-        sc = findRSDom.getSourceCode();
+        JSourceCode sc = findRSDom.getSourceCode();
         sc.add( "boolean shouldExist = ( dom != null ) && ( dom.getChildCount() > 0 || dom.getValue() != null );" );
         sc.add( "Element element = updateElement( counter, parent, name, shouldExist );" );
         sc.add( "if ( shouldExist )" );
@@ -561,51 +585,37 @@ public class JDOMWriterGenerator
         sc.addIndented( "parent.setText( parentDom.getValue() );" );
         sc.add( "}" );
 
-        return new JMethod[]{findRSElement, updateElement, insAtPref, findRSProps, findRSLists, findRSDom, findRSDom2};
+        return new JMethod[] { findRSDom, findRSDom2 };
     }
 
     private void writeAllClasses( Model objectModel, JClass jClass, ModelClass rootClass )
         throws ModelloException
     {
-        ArrayList alwaysExistingElements = new ArrayList();
+        List<ModelClass> alwaysExistingElements = new ArrayList<ModelClass>();
         alwaysExistingElements.add( rootClass );
-        for ( Iterator i = objectModel.getClasses( getGeneratedVersion() ).iterator(); i.hasNext(); )
+
+        for ( ModelClass clazz : getClasses( objectModel ) )
         {
-            ModelClass clazz = (ModelClass) i.next();
-
-            JavaClassMetadata javaClassMetadata = (JavaClassMetadata) clazz.getMetadata( JavaClassMetadata.ID );
-
-            if ( !javaClassMetadata.isEnabled() )
-            {
-                // Skip import of those classes that are not enabled for the java plugin.
-                continue;
-            }
-
             updateClass( clazz, jClass, alwaysExistingElements );
         }
     }
 
-    private void updateClass( ModelClass clazz, JClass jClass, ArrayList alwaysExisting )
+    private void updateClass( ModelClass clazz, JClass jClass, List<ModelClass> alwaysExisting )
         throws ModelloException
     {
         String className = clazz.getName();
+
+        String capClassName = capitalise( className );
+
         String uncapClassName = uncapitalise( className );
-        String clazzTagName = null;
-        if ( clazz.hasMetadata( XmlFieldMetadata.ID ) )
-        {
-            XmlFieldMetadata clazzMetadata = (XmlFieldMetadata) clazz.getMetadata( XmlFieldMetadata.ID );
-            clazzTagName = clazzMetadata.getTagName();
-        }
-        JMethod marshall = new JMethod( "update" + className );
-        marshall.addParameter( new JParameter( new JClass( className ), "value" ) );
+
+        JMethod marshall = new JMethod( "update" + capClassName );
+        marshall.addParameter( new JParameter( new JClass( className ), uncapClassName ) );
         marshall.addParameter( new JParameter( new JClass( "String" ), "xmlTag" ) );
         marshall.addParameter( new JParameter( new JClass( "Counter" ), "counter" ) );
         marshall.addParameter( new JParameter( new JClass( "Element" ), "element" ) );
         marshall.getModifiers().makeProtected();
-        if ( clazzTagName == null )
-        {
-            clazzTagName = uncapClassName;
-        }
+
         JSourceCode sc = marshall.getSourceCode();
         boolean shouldExist = alwaysExisting.contains( clazz );
         if ( shouldExist )
@@ -614,7 +624,7 @@ public class JDOMWriterGenerator
         }
         else
         {
-            sc.add( "boolean shouldExist = ( value != null );" );
+            sc.add( "boolean shouldExist = ( " + uncapClassName + " != null );" );
             sc.add( "Element root = updateElement( counter, element, xmlTag, shouldExist );" );
             sc.add( "if ( shouldExist )" );
             sc.add( "{" );
@@ -622,20 +632,17 @@ public class JDOMWriterGenerator
         }
         sc.add( "Counter innerCount = new Counter( counter.getDepth() + 1 );" );
 
-        List modelFields = clazz.getAllFields( getGeneratedVersion(), true );
+        List<ModelField> modelFields = getFieldsForXml( clazz, getGeneratedVersion() );
 
-        modelFields = getNonTransientFields( modelFields );
-
-        for ( Iterator i = modelFields.iterator(); i.hasNext(); )
+        for ( ModelField field : modelFields )
         {
-            ModelField field = (ModelField) i.next();
             XmlFieldMetadata xmlFieldMetadata = (XmlFieldMetadata) field.getMetadata( XmlFieldMetadata.ID );
             JavaFieldMetadata javaFieldMetadata = (JavaFieldMetadata) field.getMetadata( JavaFieldMetadata.ID );
 
             String fieldTagName = resolveTagName( field, xmlFieldMetadata );
 
             String type = field.getType();
-            String value = "value." + getPrefix( javaFieldMetadata ) + capitalise( field.getName() ) + "()";
+            String value = uncapClassName + '.' + getPrefix( javaFieldMetadata ) + capitalise( field.getName() ) + "()";
             if ( xmlFieldMetadata.isAttribute() )
             {
                 continue;
@@ -703,6 +710,8 @@ public class JDOMWriterGenerator
                 {
                     sc.add(
                         "findAndReplaceXpp3DOM( innerCount, root, \"" + fieldTagName + "\", (Xpp3Dom) " + value + " );" );
+
+                    requiresDomSupport = true;
                 }
                 else
                 {
