@@ -38,7 +38,6 @@ import org.codehaus.modello.plugin.java.javasource.JParameter;
 import org.codehaus.modello.plugin.java.javasource.JSourceCode;
 import org.codehaus.modello.plugin.java.javasource.JSourceWriter;
 import org.codehaus.modello.plugin.java.javasource.JType;
-import org.codehaus.modello.plugin.java.metadata.JavaClassMetadata;
 import org.codehaus.modello.plugins.xml.metadata.XmlAssociationMetadata;
 import org.codehaus.modello.plugins.xml.metadata.XmlFieldMetadata;
 import org.codehaus.modello.plugins.xml.metadata.XmlModelMetadata;
@@ -46,23 +45,27 @@ import org.codehaus.plexus.util.StringUtils;
 
 import java.io.IOException;
 import java.util.Arrays;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Properties;
 
 /**
  * @author <a href="mailto:jason@modello.org">Jason van Zyl</a>
  * @author <a href="mailto:evenisse@codehaus.org">Emmanuel Venisse</a>
- * @version $Id: StaxReaderGenerator.java 1311 2009-08-20 23:28:35Z hboutemy $
+ * @version $Id: StaxReaderGenerator.java 1480 2010-04-27 18:43:34Z bentmann $
  */
 public class StaxReaderGenerator
     extends AbstractStaxGenerator
 {
+
+    private boolean requiresDomSupport;
+
     public void generate( Model model, Properties parameters )
         throws ModelloException
     {
         initialize( model, parameters );
 
+        requiresDomSupport = false;
+
         try
         {
             generateStaxReader();
@@ -111,6 +114,7 @@ public class StaxReaderGenerator
         jClass.addImport( "java.io.Reader" );
         jClass.addImport( "java.io.File" );
         jClass.addImport( "java.io.FileInputStream" );
+        jClass.addImport( "java.io.InputStream" );
         jClass.addImport( "java.io.StringWriter" );
         jClass.addImport( "java.io.StringReader" );
         jClass.addImport( "java.io.ByteArrayInputStream" );
@@ -121,114 +125,206 @@ public class StaxReaderGenerator
         jClass.addImport( "java.util.regex.Pattern" );
         jClass.addImport( "java.util.Locale" );
         jClass.addImport( "javax.xml.stream.*" );
-        jClass.addImport( "org.codehaus.plexus.util.xml.Xpp3Dom" );
 
         addModelImports( jClass, null );
 
         // ----------------------------------------------------------------------
-        // Write the parse method which will do the unmarshalling.
+        // Write reference resolvers.
         // ----------------------------------------------------------------------
 
         ModelClass root = objectModel.getClass( objectModel.getRoot( getGeneratedVersion() ), getGeneratedVersion() );
+        JClass rootType = new JClass( root.getName() );
 
         GeneratorNode rootNode = findRequiredReferenceResolvers( root, null );
 
         writeReferenceResolvers( rootNode, jClass );
-        for ( Iterator i = rootNode.getNodesWithReferencableChildren().values().iterator(); i.hasNext(); )
+        for ( GeneratorNode node : rootNode.getNodesWithReferencableChildren().values() )
         {
-            GeneratorNode node = (GeneratorNode) i.next();
             writeReferenceResolvers( node, jClass );
         }
 
-        JClass returnType = new JClass( root.getName() );
-        JMethod method = new JMethod( "read", returnType, null );
+        // ----------------------------------------------------------------------
+        // Write the read(XMLStreamReader,boolean) method which will do the unmarshalling.
+        // ----------------------------------------------------------------------
 
-        method.addParameter( new JParameter( new JClass( "Reader" ), "reader" ) );
-        method.addParameter( new JParameter( JType.BOOLEAN, "strict" ) );
+        JMethod unmarshall = new JMethod( "read", rootType, null );
+        unmarshall.getModifiers().makePrivate();
 
-        method.addException( new JClass( "IOException" ) );
-        method.addException( new JClass( "XMLStreamException" ) );
+        unmarshall.addParameter( new JParameter( new JClass( "XMLStreamReader" ), "xmlStreamReader" ) );
+        unmarshall.addParameter( new JParameter( JType.BOOLEAN, "strict" ) );
 
-        JSourceCode sc = method.getSourceCode();
+        unmarshall.addException( new JClass( "IOException" ) );
+        unmarshall.addException( new JClass( "XMLStreamException" ) );
+
+        JSourceCode sc = unmarshall.getSourceCode();
+
+        String tagName = resolveTagName( root );
+        String className = root.getName();
+        String variableName = uncapitalise( className );
+
+        sc.add( "int eventType = xmlStreamReader.getEventType();" );
+
+        sc.add( "while ( eventType != XMLStreamConstants.END_DOCUMENT )" );
+
+        sc.add( "{" );
+        sc.indent();
+
+        sc.add( "if ( eventType == XMLStreamConstants.START_ELEMENT )" );
+
+        sc.add( "{" );
+        sc.indent();
+
+        sc.add( "if ( strict && ! \"" + tagName + "\".equals( xmlStreamReader.getLocalName() ) )" );
+
+        sc.add( "{" );
+        sc.addIndented( "throw new XMLStreamException( \"Expected root element '" + tagName + "' but "
+                        + "found '\" + xmlStreamReader.getLocalName() + \"'\", xmlStreamReader.getLocation(), null );" );
+        sc.add( "}" );
+
+        VersionDefinition versionDefinition = objectModel.getVersionDefinition();
+        if ( versionDefinition != null && "namespace".equals( versionDefinition.getType() ) )
+        {
+            sc.add( "String modelVersion = getVersionFromRootNamespace( xmlStreamReader );" );
+
+            writeModelVersionCheck( sc );
+        }
+
+        sc.add( className + ' ' + variableName + " = parse" + root.getName() + "( xmlStreamReader, strict );" );
+
+        sc.add( variableName + ".setModelEncoding( xmlStreamReader.getCharacterEncodingScheme() );" );
+
+        sc.add( "resolveReferences( " + variableName + " );" );
+
+        sc.add( "return " + variableName + ';' );
+
+        sc.unindent();
+        sc.add( "}" );
+
+        sc.add( "eventType = xmlStreamReader.next();" );
+
+        sc.unindent();
+        sc.add( "}" );
+
+        sc.add( "throw new XMLStreamException( \"Expected root element '" + tagName + "' but "
+                        + "found no element at all: invalid XML document\", xmlStreamReader.getLocation(), null );" );
+
+        jClass.addMethod( unmarshall );
+
+        // ----------------------------------------------------------------------
+        // Write the read(Reader[,boolean]) methods which will do the unmarshalling.
+        // ----------------------------------------------------------------------
+
+        unmarshall = new JMethod( "read", rootType, null );
+
+        unmarshall.addParameter( new JParameter( new JClass( "Reader" ), "reader" ) );
+        unmarshall.addParameter( new JParameter( JType.BOOLEAN, "strict" ) );
+
+        unmarshall.addException( new JClass( "IOException" ) );
+        unmarshall.addException( new JClass( "XMLStreamException" ) );
+
+        sc = unmarshall.getSourceCode();
 
         sc.add( "XMLStreamReader xmlStreamReader = XMLInputFactory.newInstance().createXMLStreamReader( reader );" );
 
         sc.add( "" );
 
-        sc.add( "String encoding = xmlStreamReader.getCharacterEncodingScheme();" );
+        sc.add( "return read( xmlStreamReader, strict );" );
 
-        sc.add( returnType + " value = parse" + root.getName() + "( \"" + resolveTagName( root )
-            + "\", xmlStreamReader, strict, encoding );" );
-
-        sc.add( "resolveReferences( value );" );
-
-        sc.add( "return value;" );
-
-        jClass.addMethod( method );
+        jClass.addMethod( unmarshall );
 
         // ----------------------------------------------------------------------
 
-        method = new JMethod( "read", returnType, null );
+        unmarshall = new JMethod( "read", rootType, null );
 
-        method.addParameter( new JParameter( new JClass( "Reader" ), "reader" ) );
+        unmarshall.addParameter( new JParameter( new JClass( "Reader" ), "reader" ) );
 
-        method.addException( new JClass( "IOException" ) );
-        method.addException( new JClass( "XMLStreamException" ) );
+        unmarshall.addException( new JClass( "IOException" ) );
+        unmarshall.addException( new JClass( "XMLStreamException" ) );
 
-        sc = method.getSourceCode();
+        sc = unmarshall.getSourceCode();
         sc.add( "return read( reader, true );" );
 
-        jClass.addMethod( method );
+        jClass.addMethod( unmarshall );
 
+        // ----------------------------------------------------------------------
+        // Write the read(InputStream[,boolean]) methods which will do the unmarshalling.
         // ----------------------------------------------------------------------
 
-        method = new JMethod( "read", returnType, null );
+        unmarshall = new JMethod( "read", rootType, null );
 
-        method.addParameter( new JParameter( new JClass( "String" ), "filePath" ) );
+        unmarshall.addParameter( new JParameter( new JClass( "InputStream" ), "stream" ) );
+        unmarshall.addParameter( new JParameter( JType.BOOLEAN, "strict" ) );
 
-        method.addParameter( new JParameter( JType.BOOLEAN, "strict" ) );
+        unmarshall.addException( new JClass( "IOException" ) );
+        unmarshall.addException( new JClass( "XMLStreamException" ) );
 
-        method.addException( new JClass( "IOException" ) );
-        method.addException( new JClass( "XMLStreamException" ) );
+        sc = unmarshall.getSourceCode();
 
-        sc = method.getSourceCode();
-
-        sc.add( "File file = new File(filePath);" );
-
-        sc.add( "XMLStreamReader xmlStreamReader = XMLInputFactory.newInstance().createXMLStreamReader( "
-                + "file.toURL().toExternalForm(), new FileInputStream(file) );" );
+        sc.add( "XMLStreamReader xmlStreamReader = XMLInputFactory.newInstance().createXMLStreamReader( stream );" );
 
         sc.add( "" );
 
-        sc.add( "String encoding = xmlStreamReader.getCharacterEncodingScheme();" );
+        sc.add( "return read( xmlStreamReader, strict );" );
 
-        sc.add( returnType + " value = parse" + root.getName() + "( \"" + resolveTagName( root )
-            + "\", xmlStreamReader, strict, encoding );" );
-
-        sc.add( "resolveReferences( value );" );
-
-        sc.add( "return value;" );
-
-        jClass.addMethod( method );
+        jClass.addMethod( unmarshall );
 
         // ----------------------------------------------------------------------
 
-        method = new JMethod( "read", returnType, null );
+        unmarshall = new JMethod( "read", rootType, null );
 
-        method.addParameter( new JParameter( new JClass( "String" ), "filePath" ) );
+        unmarshall.addParameter( new JParameter( new JClass( "InputStream" ), "stream" ) );
 
-        method.addException( new JClass( "IOException" ) );
-        method.addException( new JClass( "XMLStreamException" ) );
+        unmarshall.addException( new JClass( "IOException" ) );
+        unmarshall.addException( new JClass( "XMLStreamException" ) );
 
-        sc = method.getSourceCode();
+        sc = unmarshall.getSourceCode();
+        sc.add( "return read( stream, true );" );
+
+        jClass.addMethod( unmarshall );
+
+        // ----------------------------------------------------------------------
+        // Write the read(String[,boolean]) methods which will do the unmarshalling.
+        // ----------------------------------------------------------------------
+
+        unmarshall = new JMethod( "read", rootType, null );
+
+        unmarshall.addParameter( new JParameter( new JClass( "String" ), "filePath" ) );
+
+        unmarshall.addParameter( new JParameter( JType.BOOLEAN, "strict" ) );
+
+        unmarshall.addException( new JClass( "IOException" ) );
+        unmarshall.addException( new JClass( "XMLStreamException" ) );
+
+        sc = unmarshall.getSourceCode();
+
+        sc.add( "File file = new File( filePath );" );
+
+        sc.add( "XMLStreamReader xmlStreamReader = XMLInputFactory.newInstance().createXMLStreamReader( "
+                + "file.toURL().toExternalForm(), new FileInputStream( file ) );" );
+
+        sc.add( "" );
+
+        sc.add( "return read( xmlStreamReader, strict );" );
+
+        jClass.addMethod( unmarshall );
+
+        // ----------------------------------------------------------------------
+
+        unmarshall = new JMethod( "read", rootType, null );
+
+        unmarshall.addParameter( new JParameter( new JClass( "String" ), "filePath" ) );
+
+        unmarshall.addException( new JClass( "IOException" ) );
+        unmarshall.addException( new JClass( "XMLStreamException" ) );
+
+        sc = unmarshall.getSourceCode();
         sc.add( "return read( filePath, true );" );
 
-        jClass.addMethod( method );
+        jClass.addMethod( unmarshall );
 
         // Determine the version. Currently, it causes the document to be reparsed, but could be made more efficient in
         // future by buffering the read XML and piping that into any consequent read method.
 
-        VersionDefinition versionDefinition = objectModel.getVersionDefinition();
         if ( versionDefinition != null )
         {
             writeDetermineVersionMethod( jClass, objectModel );
@@ -246,7 +342,11 @@ public class StaxReaderGenerator
 
         writeHelpers( jClass );
 
-        writeBuildDomMethod( jClass );
+        if ( requiresDomSupport )
+        {
+            jClass.addImport( "org.codehaus.plexus.util.xml.Xpp3Dom" );
+            writeBuildDomMethod( jClass );
+        }
 
         // ----------------------------------------------------------------------
         //
@@ -257,7 +357,7 @@ public class StaxReaderGenerator
         sourceWriter.close();
     }
 
-    private void generateStaxReaderDelegate( List/*<String>*/ versions )
+    private void generateStaxReaderDelegate( List<String> versions )
         throws ModelloException, IOException
     {
         Model objectModel = getModel();
@@ -312,10 +412,8 @@ public class StaxReaderGenerator
         writeModelVersionHack( sc );
 
         String prefix = "";
-        for ( Iterator i = versions.iterator(); i.hasNext(); )
+        for ( String version : versions )
         {
-            String version = (String) i.next();
-
             sc.add( prefix + "if ( \"" + version + "\".equals( modelVersion ) )" );
             sc.add( "{" );
             sc.addIndented( "return new " + getModel().getDefaultPackageName( true, new Version( version ) )
@@ -362,7 +460,10 @@ public class StaxReaderGenerator
     private static void writeModelVersionHack( JSourceCode sc )
     {
         sc.add( "// legacy hack for pomVersion == 3" );
-        sc.add( "if ( modelVersion.equals( \"3\" ) ) modelVersion = \"3.0.0\";" );
+        sc.add( "if ( \"3\".equals( modelVersion ) )" );
+        sc.add( "{" );
+        sc.addIndented( "modelVersion = \"3.0.0\";" );
+        sc.add( "}" );
     }
 
     private void writeDetermineVersionMethod( JClass jClass, Model objectModel )
@@ -552,18 +653,8 @@ public class StaxReaderGenerator
     {
         ModelClass root = objectModel.getClass( objectModel.getRoot( getGeneratedVersion() ), getGeneratedVersion() );
 
-        for ( Iterator i = objectModel.getClasses( getGeneratedVersion() ).iterator(); i.hasNext(); )
+        for ( ModelClass clazz : getClasses( objectModel ) )
         {
-            ModelClass clazz = (ModelClass) i.next();
-
-            JavaClassMetadata javaClassMetadata = (JavaClassMetadata) clazz.getMetadata( JavaClassMetadata.ID );
-
-            if ( !javaClassMetadata.isEnabled() )
-            {
-                // Skip import of those classes that are not enabled for the java plugin.
-                continue;
-            }
-
             writeClassParser( clazz, jClass, root.getName().equals( clazz.getName() ) );
         }
     }
@@ -588,10 +679,8 @@ public class StaxReaderGenerator
         JMethod unmarshall = new JMethod( "parse" + capClassName, new JClass( className ), null );
         unmarshall.getModifiers().makePrivate();
 
-        unmarshall.addParameter( new JParameter( new JClass( "String" ), "tagName" ) );
         unmarshall.addParameter( new JParameter( new JClass( "XMLStreamReader" ), "xmlStreamReader" ) );
         unmarshall.addParameter( new JParameter( JType.BOOLEAN, "strict" ) );
-        unmarshall.addParameter( new JParameter( new JClass( "String" ), "encoding" ) );
 
         unmarshall.addException( new JClass( "IOException" ) );
         unmarshall.addException( new JClass( "XMLStreamException" ) );
@@ -615,81 +704,35 @@ public class StaxReaderGenerator
 
             String instanceFieldName = getInstanceFieldName( className );
 
-            if ( rootElement )
+            writeAttributes( modelClass, uncapClassName, sc );
+
+            if ( isAssociationPartToClass( modelClass ) )
             {
-                // encoding parameter is only used in root class
-                sc.add( uncapClassName + ".setModelEncoding( encoding );" );
-
-                sc.add( "boolean foundRoot = false;" );
-
-                sc.add( "while ( xmlStreamReader.hasNext() )" );
+                jClass.addField( new JField( new JType( "java.util.Map" ), instanceFieldName ) );
 
+                sc.add( "if ( " + instanceFieldName + " == null )" );
                 sc.add( "{" );
-                sc.indent();
+                sc.addIndented( instanceFieldName + " = new java.util.HashMap();" );
+                sc.add( "}" );
 
-                sc.add( "int eventType = xmlStreamReader.next();" );
-
-                sc.add( "if ( eventType == XMLStreamConstants.START_ELEMENT )" );
+                sc.add( "String v = xmlStreamReader.getAttributeValue( null, \"modello.id\" );" );
+                sc.add( "if ( v != null )" );
+                sc.add( "{" );
+                sc.addIndented( instanceFieldName + ".put( v, " + uncapClassName + " );" );
+                sc.add( "}" );
             }
-            else
-            {
-                writeAttributes( modelClass, uncapClassName, sc );
 
-                if ( isAssociationPartToClass( modelClass ) )
-                {
-                    jClass.addField( new JField( new JType( "java.util.Map" ), instanceFieldName ) );
-
-                    sc.add( "if ( " + instanceFieldName + " == null )" );
-                    sc.add( "{" );
-                    sc.addIndented( instanceFieldName + " = new java.util.HashMap();" );
-                    sc.add( "}" );
-
-                    sc.add( "String v = xmlStreamReader.getAttributeValue( null, \"modello.id\" );" );
-                    sc.add( "if ( v != null )" );
-                    sc.add( "{" );
-                    sc.addIndented( instanceFieldName + ".put( v, " + uncapClassName + " );" );
-                    sc.add( "}" );
-                }
-
-                sc.add( "while ( xmlStreamReader.nextTag() == XMLStreamConstants.START_ELEMENT )" );
-            }
+            sc.add( "while ( ( strict ? xmlStreamReader.nextTag() : nextTag( xmlStreamReader ) ) == XMLStreamConstants.START_ELEMENT )" );
 
             sc.add( "{" );
             sc.indent();
 
             boolean addElse = false;
 
-            if ( rootElement )
-            {
-                sc.add( "if ( xmlStreamReader.getLocalName().equals( tagName ) )" );
-
-                sc.add( "{" );
-                sc.indent();
-
-                VersionDefinition versionDefinition = modelClass.getModel().getVersionDefinition();
-                if ( versionDefinition != null && "namespace".equals( versionDefinition.getType() ) )
-                {
-                    sc.add( "String modelVersion = getVersionFromRootNamespace( xmlStreamReader );" );
-
-                    writeModelVersionCheck( sc );
-                }
-
-                writeAttributes( modelClass, uncapClassName, sc );
-
-                sc.add( "foundRoot = true;" );
-
-                sc.unindent();
-                sc.add( "}" );
-
-                addElse = true;
-            }
-
             // Write other fields
 
-            for ( Iterator i = modelClass.getAllFields( getGeneratedVersion(), true ).iterator(); i.hasNext(); )
+            for ( ModelField field : modelClass.getAllFields( getGeneratedVersion(), true ) )
             {
-                ModelField field = (ModelField) i.next();
-
                 XmlFieldMetadata xmlFieldMetadata = (XmlFieldMetadata) field.getMetadata( XmlFieldMetadata.ID );
 
                 if ( !xmlFieldMetadata.isAttribute() && !xmlFieldMetadata.isTransient() )
@@ -699,94 +742,30 @@ public class StaxReaderGenerator
                     addElse = true;
                 }
             }
-            if ( !rootElement )
+
+            /*
+            if ( modelClass.getFields( getGeneratedVersion() ).size() > 0 )
             {
-    /*
-                if ( modelClass.getFields( getGeneratedVersion() ).size() > 0 )
-                {
-                    sc.add( "else" );
-
-                    sc.add( "{" );
-                    sc.addIndented( "parser.nextText();" );
-                    sc.add( "}" );
-                }
-    */
-
-                if ( addElse )
-                {
-                    sc.add( "else" );
-
-                    sc.add( "{" );
-                    sc.indent();
-                }
-
-                sc.add( "if ( strict )" );
-
-                sc.add( "{" );
-                sc.addIndented( "throw new XMLStreamException( \"Unrecognised tag: '\" + xmlStreamReader.getLocalName() + "
-                                + "\"'\", xmlStreamReader.getLocation() );" );
-                sc.add( "}" );
-
                 sc.add( "else" );
 
                 sc.add( "{" );
-                sc.indent();
-
-                sc.add( "int unrecognizedTagCount = 1;" );
-                sc.add( "while( unrecognizedTagCount != 0 )" );
-
-                sc.add( "{" );
-                sc.indent();
-
-                sc.add( "xmlStreamReader.next();" );
-                sc.add( "if ( xmlStreamReader.getEventType() == XMLStreamConstants.START_ELEMENT )" );
-
-                sc.add( "{" );
-                sc.addIndented( "unrecognizedTagCount++;" );
+                sc.addIndented( "parser.nextText();" );
                 sc.add( "}" );
-
-                sc.add( "else if ( xmlStreamReader.getEventType() == XMLStreamConstants.END_ELEMENT )" );
-                sc.add( "{" );
-                sc.addIndented( "unrecognizedTagCount--;" );
-                sc.add( "}" );
-
-                sc.unindent();
-                sc.add( "}" );
-
-                sc.unindent();
-                sc.add( "}" );
-
-                if ( addElse )
-                {
-                    sc.unindent();
-                    sc.add( "}" );
-                }
             }
-            else
-            { // rootElement == true
+*/
+
+            if ( addElse )
+            {
                 sc.add( "else" );
 
                 sc.add( "{" );
                 sc.indent();
+            }
 
-                sc.add( "if ( foundRoot )" );
-
-                sc.add( "{" );
-                sc.indent();
-
-                sc.add( "if ( strict )" );
-
-                sc.add( "{" );
-                sc.addIndented( "throw new XMLStreamException( \"Unrecognised tag: '\" + xmlStreamReader.getLocalName() + "
-                                + "\"'\", xmlStreamReader.getLocation() );" );
-                sc.add( "}" );
-
-                sc.unindent();
-                sc.add( "}" );
-
-                sc.unindent();
-                sc.add( "}" );
+            sc.add( "checkUnknownElement( xmlStreamReader, strict );" );
 
+            if ( addElse )
+            {
                 sc.unindent();
                 sc.add( "}" );
             }
@@ -797,7 +776,7 @@ public class StaxReaderGenerator
             // This must be last so that we guarantee the ID has been filled already
             if ( isAssociationPartToClass( modelClass ) )
             {
-                List identifierFields = modelClass.getIdentifierFields( getGeneratedVersion() );
+                List<ModelField> identifierFields = modelClass.getIdentifierFields( getGeneratedVersion() );
 
                 if ( identifierFields.size() == 1 )
                 {
@@ -822,10 +801,8 @@ public class StaxReaderGenerator
 
         GeneratorNode value = new GeneratorNode( className, parent );
 
-        for ( Iterator i = modelClass.getAllFields( getGeneratedVersion(), true ).iterator(); i.hasNext(); )
+        for ( ModelField field : modelClass.getAllFields( getGeneratedVersion(), true ) )
         {
-            ModelField field = (ModelField) i.next();
-
             if ( field instanceof ModelAssociation )
             {
                 ModelAssociation association = (ModelAssociation) field;
@@ -858,10 +835,8 @@ public class StaxReaderGenerator
         }
 
         // propagate the flag up
-        for ( Iterator i = value.getChildren().iterator(); i.hasNext(); )
+        for ( GeneratorNode child : value.getChildren() )
         {
-            GeneratorNode child = (GeneratorNode) i.next();
-
             if ( child.isReferencable() || child.isReferencableChildren() )
             {
                 value.setReferencableChildren( true );
@@ -885,10 +860,8 @@ public class StaxReaderGenerator
 
         sc.add( "java.util.Map refs;" );
 
-        for ( Iterator i = node.getChildren().iterator(); i.hasNext(); )
+        for ( GeneratorNode child : node.getChildren() )
         {
-            GeneratorNode child = (GeneratorNode) i.next();
-
             if ( child.isReferencable() )
             {
                 ModelAssociation association = child.getAssociation();
@@ -986,10 +959,8 @@ public class StaxReaderGenerator
     private void writeAttributes( ModelClass modelClass, String uncapClassName, JSourceCode sc )
         throws ModelloException
     {
-        for ( Iterator i = modelClass.getAllFields( getGeneratedVersion(), true ).iterator(); i.hasNext(); )
+        for ( ModelField field : modelClass.getAllFields( getGeneratedVersion(), true ) )
         {
-            ModelField field = (ModelField) i.next();
-
             XmlFieldMetadata xmlFieldMetadata = (XmlFieldMetadata) field.getMetadata( XmlFieldMetadata.ID );
 
             if ( xmlFieldMetadata.isAttribute() && !xmlFieldMetadata.isTransient() )
@@ -1082,8 +1053,8 @@ public class StaxReaderGenerator
                 }
                 else
                 {
-                    sc.add( objectName + ".set" + capFieldName + "( parse" + association.getTo() + "( \"" + fieldTagName
-                            + "\", xmlStreamReader, strict, encoding ) );" );
+                    sc.add( objectName + ".set" + capFieldName + "( parse" + association.getTo()
+                        + "( xmlStreamReader, strict ) );" );
                 }
 
                 sc.unindent();
@@ -1120,7 +1091,7 @@ public class StaxReaderGenerator
                         sc.add( "{" );
                         sc.indent();
 
-                        sc.add( "if ( xmlStreamReader.getLocalName().equals( \"" + valuesTagName + "\" ) )" );
+                        sc.add( "if ( \"" + valuesTagName + "\".equals( xmlStreamReader.getLocalName() ) )" );
 
                         sc.add( "{" );
                         sc.indent();
@@ -1128,7 +1099,7 @@ public class StaxReaderGenerator
                     else
                     {
                         sc.add( ( addElse ? "else " : "" )
-                            + "if ( xmlStreamReader.getLocalName().equals( \"" + valuesTagName + "\" ) )" );
+                            + "if ( \"" + valuesTagName + "\".equals( xmlStreamReader.getLocalName() ) )" );
 
                         sc.add( "{" );
                         sc.indent();
@@ -1162,14 +1133,13 @@ public class StaxReaderGenerator
                             // HACK: the addXXX method will cause an OOME when compiling a self-referencing class, so we
                             //  just add it to the array. This could disrupt the links if you are using break/create
                             //  constraints in modello.
-                            sc.add( associationName + ".add( parse" + association.getTo() + "( \"" + valuesTagName
-                                    + "\", xmlStreamReader, strict, encoding ) );" );
+                            sc.add( associationName + ".add( parse" + association.getTo()
+                                + "( xmlStreamReader, strict ) );" );
                         }
                         else
                         {
                             sc.add( objectName + ".add" + capitalise( singular( associationName ) ) + "( parse"
-                                    + association.getTo() + "( \"" + valuesTagName
-                                    + "\", xmlStreamReader, strict, encoding ) );" );
+                                    + association.getTo() + "( xmlStreamReader, strict ) );" );
                         }
                     }
                     else
@@ -1218,7 +1188,7 @@ public class StaxReaderGenerator
                         sc.add( "{" );
                         sc.indent();
 
-                        sc.add( "if ( xmlStreamReader.getLocalName().equals( \"" + valuesTagName + "\" ) )" );
+                        sc.add( "if ( \"" + valuesTagName + "\".equals( xmlStreamReader.getLocalName() ) )" );
 
                         sc.add( "{" );
                         sc.indent();
@@ -1234,13 +1204,13 @@ public class StaxReaderGenerator
                         sc.add( "{" );
                         sc.indent();
 
-                        sc.add( "if ( xmlStreamReader.getLocalName().equals( \"key\" ) )" );
+                        sc.add( "if ( \"key\".equals( xmlStreamReader.getLocalName() ) )" );
 
                         sc.add( "{" );
                         sc.addIndented( "key = xmlStreamReader.getElementText();" );
                         sc.add( "}" );
 
-                        sc.add( "else if ( xmlStreamReader.getLocalName().equals( \"value\" ) )" );
+                        sc.add( "else if ( \"value\".equals( xmlStreamReader.getLocalName() ) )" );
 
                         sc.add( "{" );
                         sc.addIndented( "value = xmlStreamReader.getElementText()"
@@ -1342,7 +1312,7 @@ public class StaxReaderGenerator
     {
         writeModelVersionHack( sc );
 
-        sc.add( "if ( !modelVersion.equals( \"" + getGeneratedVersion() + "\" ) )" );
+        sc.add( "if ( !\"" + getGeneratedVersion() + "\".equals( modelVersion ) )" );
         sc.add( "{" );
         sc.addIndented(
             "throw new XMLStreamException( \"Document model version of '\" + modelVersion + \"' doesn't match reader "
@@ -1380,7 +1350,7 @@ public class StaxReaderGenerator
 /* TODO:
         if ( xmlFieldMetadata.isRequired() )
         {
-            parserGetter = "getRequiredAttributeValue( " + parserGetter + ", \"" + tagName + "\", parser, strict, encoding )";
+            parserGetter = "getRequiredAttributeValue( " + parserGetter + ", \"" + tagName + "\", parser, strict )";
         }
 */
         if ( field.getDefaultValue() != null )
@@ -1448,6 +1418,8 @@ public class StaxReaderGenerator
         else if ( "DOM".equals( type ) )
         {
             sc.add( objectName + "." + setterName + "( buildDom( xmlStreamReader ) );" );
+
+            requiresDomSupport = true;
         }
         else
         {
@@ -1737,7 +1709,7 @@ public class StaxReaderGenerator
         method.addParameter( new JParameter( new JClass( "XMLStreamReader" ), "xmlStreamReader" ) );
         method.addException( new JClass( "XMLStreamException" ) );
 
-        writeDateParsingHelper( method.getSourceCode(), "new XMLStreamException( e.getMessage() )" );
+        writeDateParsingHelper( method.getSourceCode(), "new XMLStreamException( e.getMessage(), xmlStreamReader.getLocation(), e )" );
 
         jClass.addMethod( method );
 
@@ -1761,18 +1733,92 @@ public class StaxReaderGenerator
         sc.addIndented( "return false;" );
         sc.add( "}" );
 
-        sc.add( "if ( parsed.contains( tagName ) )" );
+        sc.add( "if ( !parsed.add( tagName ) )" );
 
         sc.add( "{" );
         sc.addIndented(
             "throw new XMLStreamException( \"Duplicated tag: '\" + tagName + \"'\", xmlStreamReader.getLocation() );" );
         sc.add( "}" );
 
-        sc.add( "parsed.add( tagName );" );
-
         sc.add( "return true;" );
 
         jClass.addMethod( method );
+
+        // --------------------------------------------------------------------
+
+        method = new JMethod( "checkUnknownElement", null, null );
+        method.getModifiers().makePrivate();
+
+        method.addParameter( new JParameter( new JClass( "XMLStreamReader" ), "xmlStreamReader" ) );
+        method.addParameter( new JParameter( JType.BOOLEAN, "strict" ) );
+        method.addException( new JClass( "XMLStreamException" ) );
+
+        sc = method.getSourceCode();
+
+        sc.add( "if ( strict )" );
+
+        sc.add( "{" );
+        sc.addIndented( "throw new XMLStreamException( \"Unrecognised tag: '\" + xmlStreamReader.getLocalName() + "
+                        + "\"'\", xmlStreamReader.getLocation() );" );
+        sc.add( "}" );
+
+        sc.add( "int unrecognizedTagCount = 1;" );
+        sc.add( "while( unrecognizedTagCount != 0 )" );
+
+        sc.add( "{" );
+        sc.indent();
+
+        sc.add( "xmlStreamReader.next();" );
+        sc.add( "if ( xmlStreamReader.getEventType() == XMLStreamConstants.START_ELEMENT )" );
+
+        sc.add( "{" );
+        sc.addIndented( "unrecognizedTagCount++;" );
+        sc.add( "}" );
+
+        sc.add( "else if ( xmlStreamReader.getEventType() == XMLStreamConstants.END_ELEMENT )" );
+        sc.add( "{" );
+        sc.addIndented( "unrecognizedTagCount--;" );
+        sc.add( "}" );
+
+        sc.unindent();
+        sc.add( "}" );
+
+        jClass.addMethod( method );
+
+        // --------------------------------------------------------------------
+
+        method = new JMethod( "nextTag", JType.INT, null );
+        method.getModifiers().makePrivate();
+
+        method.addParameter( new JParameter( new JClass( "XMLStreamReader" ), "xmlStreamReader" ) );
+        method.addException( new JClass( "XMLStreamException" ) );
+
+        sc = method.getSourceCode();
+
+        sc.add( "while ( true )" );
+        sc.add( "{" );
+        sc.indent();
+        sc.add( "int eventType = xmlStreamReader.next();" );
+        sc.add( "switch ( eventType )" );
+        sc.add( "{" );
+        sc.indent();
+        sc.add( "case XMLStreamConstants.CHARACTERS:" );
+        sc.add( "case XMLStreamConstants.CDATA:" );
+        sc.add( "case XMLStreamConstants.SPACE:" );
+        sc.add( "case XMLStreamConstants.PROCESSING_INSTRUCTION:" );
+        sc.add( "case XMLStreamConstants.COMMENT:" );
+        sc.addIndented( "break;" );
+        sc.add( "case XMLStreamConstants.START_ELEMENT:" );
+        sc.add( "case XMLStreamConstants.END_ELEMENT:" );
+        sc.addIndented( "return eventType;" );
+        sc.add( "default:" );
+        sc.addIndented( "throw new XMLStreamException( \"expected start or end tag\", xmlStreamReader.getLocation() );" );
+        sc.unindent();
+        sc.add( "}" );
+        sc.unindent();
+        sc.add( "}" );
+
+        jClass.addMethod( method );
     }
 
     private JMethod convertNumericalType( String methodName, JType returnType, String expression, String typeDesc )
@@ -1799,7 +1845,7 @@ public class StaxReaderGenerator
         sc.addIndented( "return " + expression + ";" );
         sc.add( "}" );
 
-        sc.add( "catch ( NumberFormatException e )" );
+        sc.add( "catch ( NumberFormatException nfe )" );
 
         sc.add( "{" );
         sc.indent();
@@ -1808,7 +1854,7 @@ public class StaxReaderGenerator
 
         sc.add( "{" );
         sc.addIndented( "throw new XMLStreamException( \"Unable to parse element '\" + attribute + \"', must be "
-                        + typeDesc + " but was '\" + s + \"'\", xmlStreamReader.getLocation() );" );
+                        + typeDesc + " but was '\" + s + \"'\", xmlStreamReader.getLocation(), nfe );" );
         sc.add( "}" );
 
         sc.unindent();
