@@ -36,7 +36,6 @@ import org.codehaus.modello.plugin.java.javasource.JParameter;
 import org.codehaus.modello.plugin.java.javasource.JSourceCode;
 import org.codehaus.modello.plugin.java.javasource.JSourceWriter;
 import org.codehaus.modello.plugin.java.javasource.JType;
-import org.codehaus.modello.plugin.java.metadata.JavaClassMetadata;
 import org.codehaus.modello.plugin.java.metadata.JavaFieldMetadata;
 import org.codehaus.modello.plugin.model.ModelClassMetadata;
 import org.codehaus.modello.plugins.xml.metadata.XmlAssociationMetadata;
@@ -44,23 +43,29 @@ import org.codehaus.modello.plugins.xml.metadata.XmlFieldMetadata;
 import org.codehaus.modello.plugins.xml.metadata.XmlModelMetadata;
 
 import java.io.IOException;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Properties;
 
 /**
  * @author <a href="mailto:jason@modello.org">Jason van Zyl </a>
  * @author <a href="mailto:evenisse@codehaus.org">Emmanuel Venisse </a>
- * @version $Id: StaxWriterGenerator.java 1311 2009-08-20 23:28:35Z hboutemy $
+ * @version $Id: StaxWriterGenerator.java 1474 2010-04-24 19:59:00Z bentmann $
  */
 public class StaxWriterGenerator
     extends AbstractStaxGenerator
 {
+
+    private boolean requiresDomSupport;
+
+    private StaxSerializerGenerator serializerGenerator;
+
     public void generate( Model model, Properties parameters )
         throws ModelloException
     {
         initialize( model, parameters );
 
+        requiresDomSupport = false;
+
         try
         {
             generateStaxWriter();
@@ -69,6 +74,8 @@ public class StaxWriterGenerator
         {
             throw new ModelloException( "Exception while generating StAX Writer.", ex );
         }
+        
+        serializerGenerator.generate( model, parameters );
     }
 
     private void generateStaxWriter()
@@ -87,8 +94,8 @@ public class StaxWriterGenerator
         initHeader( jClass );
         suppressAllWarnings( objectModel, jClass );
 
-        jClass.addImport( "java.io.InputStream" );
         jClass.addImport( "java.io.IOException" );
+        jClass.addImport( "java.io.OutputStream" );
         jClass.addImport( "java.io.Writer" );
         jClass.addImport( "java.io.StringWriter" );
         jClass.addImport( "java.text.DateFormat" );
@@ -96,8 +103,6 @@ public class StaxWriterGenerator
         jClass.addImport( "java.util.Locale" );
         jClass.addImport( "java.util.jar.Manifest" );
         jClass.addImport( "javax.xml.stream.*" );
-        jClass.addImport( "javanet.staxutils.IndentingXMLStreamWriter" );
-        jClass.addImport( "org.codehaus.plexus.util.xml.Xpp3Dom" );
 
         addModelImports( jClass, null );
 
@@ -113,7 +118,9 @@ public class StaxWriterGenerator
 
         String rootElement = resolveTagName( rootClass );
 
-        // Write the parse method which will do the unmarshalling.
+        // ----------------------------------------------------------------------
+        // Write the write( Writer, Model ) method which will do the unmarshalling.
+        // ----------------------------------------------------------------------
 
         JMethod marshall = new JMethod( "write" );
 
@@ -160,10 +167,62 @@ public class StaxWriterGenerator
 
         jClass.addMethod( marshall );
 
-        createWriteDomMethod( jClass );
+        // ----------------------------------------------------------------------
+        // Write the write( OutputStream, Model ) method which will do the unmarshalling.
+        // ----------------------------------------------------------------------
+
+        marshall = new JMethod( "write" );
+
+        marshall.addParameter( new JParameter( new JClass( "OutputStream" ), "stream" ) );
+        marshall.addParameter( new JParameter( new JClass( root ), rootElementParameterName ) );
+
+        marshall.addException( new JClass( "java.io.IOException" ) );
+        marshall.addException( new JClass( "XMLStreamException" ) );
+
+        sc = marshall.getSourceCode();
+
+        sc.add( "XMLOutputFactory factory = XMLOutputFactory.newInstance();" );
+
+        // currently, only woodstox supports Windows line endings. It works with Java 6/RI and stax <= 1.1.1 as well
+        // but we have no way to detect them
+        sc.add( "boolean supportWindowsLineEndings = false;" );
+        sc.add( "if ( factory.isPropertySupported( \"com.ctc.wstx.outputEscapeCr\" ) )" );
+        sc.add( "{" );
+        sc.indent();
+        sc.add( "factory.setProperty( \"com.ctc.wstx.outputEscapeCr\", Boolean.FALSE );" );
+        sc.add( "supportWindowsLineEndings = true;" );
+        sc.unindent();
+        sc.add( "}" );
+
+        sc.add( "if ( factory.isPropertySupported( \"org.codehaus.stax2.automaticEmptyElements\" ) )" );
+        sc.add( "{" );
+        sc.addIndented( "factory.setProperty( \"org.codehaus.stax2.automaticEmptyElements\", Boolean.FALSE );" );
+        sc.add( "}" );
+
+        sc.add( "IndentingXMLStreamWriter serializer = new IndentingXMLStreamWriter( factory.createXMLStreamWriter( stream, "
+            + rootElementParameterName + ".getModelEncoding() ) );" );
+
+        sc.add( "if ( supportWindowsLineEndings )" );
+        sc.add( "{" );
+        sc.addIndented( "serializer.setNewLine( serializer.getLineSeparator() );" );
+        sc.add( "}" );
+
+        sc.add( "serializer.writeStartDocument( " + rootElementParameterName + ".getModelEncoding(), \"1.0\" );" );
+
+        sc.add( "write" + root + "( " + rootElementParameterName + ", \"" + rootElement + "\", serializer );" );
+
+        sc.add( "serializer.writeEndDocument();" );
+
+        jClass.addMethod( marshall );
 
         writeAllClasses( objectModel, jClass );
 
+        if ( requiresDomSupport )
+        {
+            jClass.addImport( "org.codehaus.plexus.util.xml.Xpp3Dom" );
+            createWriteDomMethod( jClass );
+        }
+
         jClass.print( sourceWriter );
 
         sourceWriter.close();
@@ -172,18 +231,8 @@ public class StaxWriterGenerator
     private void writeAllClasses( Model objectModel, JClass jClass )
         throws ModelloException
     {
-        for ( Iterator i = objectModel.getClasses( getGeneratedVersion() ).iterator(); i.hasNext(); )
+        for ( ModelClass clazz : getClasses( objectModel ) )
         {
-            ModelClass clazz = (ModelClass) i.next();
-
-            JavaClassMetadata javaClassMetadata = (JavaClassMetadata) clazz.getMetadata( JavaClassMetadata.ID );
-
-            if ( !javaClassMetadata.isEnabled() )
-            {
-                // Skip import of those classes that are not enabled for the java plugin.
-                continue;
-            }
-
             writeClass( clazz, jClass );
         }
     }
@@ -254,15 +303,11 @@ public class StaxWriterGenerator
 
         String contentValue = null;
 
-        List modelFields = modelClass.getAllFields( getGeneratedVersion(), true );
-
-        modelFields = getNonTransientFields( modelFields );
+        List<ModelField> modelFields = getFieldsForXml( modelClass, getGeneratedVersion() );
 
         // XML attributes
-        for ( Iterator i = modelFields.iterator(); i.hasNext(); )
+        for ( ModelField field : modelFields )
         {
-            ModelField field = (ModelField) i.next();
-
             XmlFieldMetadata xmlFieldMetadata = (XmlFieldMetadata) field.getMetadata( XmlFieldMetadata.ID );
 
             String fieldTagName = resolveTagName( field, xmlFieldMetadata );
@@ -296,10 +341,8 @@ public class StaxWriterGenerator
         }
 
         // XML tags
-        for ( Iterator fieldIterator = modelFields.iterator(); fieldIterator.hasNext(); )
+        for ( ModelField field : modelFields )
         {
-            ModelField field = (ModelField) fieldIterator.next();
-
             XmlFieldMetadata xmlFieldMetadata = (XmlFieldMetadata) field.getMetadata( XmlFieldMetadata.ID );
 
             if ( xmlFieldMetadata.isContent() )
@@ -486,6 +529,8 @@ public class StaxWriterGenerator
                 if ( "DOM".equals( field.getType() ) )
                 {
                     sc.add( "writeDom( (Xpp3Dom) " + value + ", serializer );" );
+
+                    requiresDomSupport = true;
                 }
                 else
                 {
