@@ -4,14 +4,13 @@ import org.codehaus.modello.model.Model;
 import org.codehaus.modello.model.ModelClass;
 import org.codehaus.modello.model.Version;
 import org.codehaus.plexus.component.repository.exception.ComponentLookupException;
-import org.codehaus.plexus.util.ReaderFactory;
 
 import java.util.List;
 
 /**
  * @author <a href="mailto:trygvis@inamo.no">Trygve Laugst&oslash;l</a>
  * @author <a href="mailto:evenisse@codehaus.org">Emmanuel Venisse</a>
- * @version $Id: StaxGeneratorWrongVersionTest.java 1094 2009-01-02 22:06:17Z hboutemy $
+ * @version $Id: StaxGeneratorWrongVersionTest.java 1413 2010-02-13 21:23:01Z hboutemy $
  */
 public class StaxGeneratorWrongVersionTest
     extends AbstractStaxGeneratorTestCase
@@ -25,9 +24,9 @@ public class StaxGeneratorWrongVersionTest
     public void testStaxReaderVersionInField()
         throws Throwable
     {
-        Model model = modello.loadModel( ReaderFactory.newXmlReader( getTestFile( "src/test/resources/version-in-field.mdo" ) ) );
+        Model model = modello.loadModel( getXmlResourceReader( "/version-in-field.mdo" ) );
 
-        List classesList = model.getClasses( new Version( "4.0.0" ) );
+        List<ModelClass> classesList = model.getClasses( new Version( "4.0.0" ) );
 
         assertEquals( 1, classesList.size() );
 
