@@ -25,9 +25,7 @@ package org.codehaus.modello.plugin.xdoc;
 import java.io.File;
 import java.io.IOException;
 import java.io.Writer;
-import java.util.ArrayList;
 import java.util.HashSet;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Properties;
 import java.util.Set;
@@ -45,10 +43,13 @@ import org.codehaus.modello.model.ModelField;
 import org.codehaus.modello.model.Version;
 import org.codehaus.modello.model.VersionRange;
 import org.codehaus.modello.plugin.xdoc.metadata.XdocFieldMetadata;
+import org.codehaus.modello.plugin.xsd.XsdModelHelper;
 import org.codehaus.modello.plugins.xml.AbstractXmlGenerator;
 import org.codehaus.modello.plugins.xml.metadata.XmlAssociationMetadata;
 import org.codehaus.modello.plugins.xml.metadata.XmlClassMetadata;
 import org.codehaus.modello.plugins.xml.metadata.XmlFieldMetadata;
+import org.codehaus.modello.plugins.xml.metadata.XmlModelMetadata;
+import org.codehaus.plexus.util.StringUtils;
 import org.codehaus.plexus.util.WriterFactory;
 import org.codehaus.plexus.util.xml.PrettyPrintXMLWriter;
 import org.codehaus.plexus.util.xml.XMLWriter;
@@ -56,7 +57,7 @@ import org.codehaus.plexus.util.xml.XMLWriter;
 /**
  * @author <a href="mailto:jason@modello.org">Jason van Zyl</a>
  * @author <a href="mailto:emmanuel@venisse.net">Emmanuel Venisse</a>
- * @version $Id: XdocGenerator.java 1275 2009-07-04 21:32:31Z bentmann $
+ * @version $Id: XdocGenerator.java 1469 2010-04-20 16:45:36Z hboutemy $
  */
 public class XdocGenerator
     extends AbstractXmlGenerator
@@ -184,7 +185,7 @@ public class XdocGenerator
      */
     private void writeModelDescriptor( XMLWriter w, ModelClass rootModelClass )
     {
-        writeElementDescriptor( w, rootModelClass, null, new HashSet() );
+        writeElementDescriptor( w, rootModelClass, null, new HashSet<String>() );
     }
 
     /**
@@ -195,11 +196,12 @@ public class XdocGenerator
      * @param association the association we are coming from (can be <code>null</code>)
      * @param written set of data already written
      */
-    private void writeElementDescriptor( XMLWriter w, ModelClass modelClass, ModelAssociation association, Set written )
+    private void writeElementDescriptor( XMLWriter w, ModelClass modelClass, ModelAssociation association,
+                                         Set<String> written )
     {
         String tagName = resolveTagName( modelClass, association );
 
-        String id = tagName + '/' + modelClass.getPackageName() + '.' + modelClass.getName();
+        String id = getId( tagName, modelClass );
         if ( written.contains( id ) )
         {
             // tag already written for this model class accessed as this tag name
@@ -221,7 +223,9 @@ public class XdocGenerator
 
         writeMarkupElement( w, "p", getDescription( modelClass ) );
 
-        ModelField contentField = getContentField( getNonTransientFields( getFieldsForClass( modelClass ) ) );
+        List<ModelField> elementFields = getFieldsForXml( modelClass, getGeneratedVersion() );
+
+        ModelField contentField = getContentField( elementFields );
 
         if ( contentField != null )
         {
@@ -235,31 +239,24 @@ public class XdocGenerator
             w.endElement();
         }
 
-        List attributeFields = new ArrayList( getAttributeFieldsForClass( modelClass ) );
+        List<ModelField> attributeFields = getXmlAttributeFields( elementFields );
 
-        attributeFields = getNonTransientFields( attributeFields );
-
-        List elementFields = new ArrayList( getFieldsForClass( modelClass ) );
         elementFields.removeAll( attributeFields );
 
-        elementFields = getNonTransientFields( elementFields );
-
         writeFieldsTable( w, attributeFields, false ); // write attributes
         writeFieldsTable( w, elementFields, true ); // write elements
 
         w.endElement();
 
         // check every fields that are inner associations to write their element descriptor
-        for ( Iterator iter = elementFields.iterator(); iter.hasNext(); )
+        for ( ModelField f : elementFields )
         {
-            ModelField f = (ModelField) iter.next();
-
             if ( isInnerAssociation( f ) )
             {
                 ModelAssociation assoc = (ModelAssociation) f;
                 ModelClass fieldModelClass = getModel().getClass( assoc.getTo(), getGeneratedVersion() );
 
-                if ( !written.contains( resolveTagName( fieldModelClass, assoc ) ) )
+                if ( !written.contains( getId( resolveTagName( fieldModelClass, assoc ), fieldModelClass ) ) )
                 {
                     writeElementDescriptor( w, fieldModelClass, assoc, written );
                 }
@@ -267,6 +264,11 @@ public class XdocGenerator
         }
     }
 
+    private String getId( String tagName, ModelClass modelClass )
+    {
+        return tagName + '/' + modelClass.getPackageName() + '.' + modelClass.getName();
+    }
+
     /**
      * Write a table containing model fields description.
      *
@@ -274,7 +276,7 @@ public class XdocGenerator
      * @param fields the fields to add in the table
      * @param elementFields <code>true</code> if fields are elements, <code>false</code> if fields are attributes
      */
-    private void writeFieldsTable( XMLWriter w, List fields, boolean elementFields )
+    private void writeFieldsTable( XMLWriter w, List<ModelField> fields, boolean elementFields )
     {
         if ( fields == null || fields.isEmpty() )
         {
@@ -307,10 +309,8 @@ public class XdocGenerator
 
         w.endElement(); // tr
 
-        for ( Iterator j = fields.iterator(); j.hasNext(); )
+        for ( ModelField f : fields )
         {
-            ModelField f = (ModelField) j.next();
-
             XmlFieldMetadata xmlFieldMetadata = (XmlFieldMetadata) f.getMetadata( XmlFieldMetadata.ID );
 
             if ( xmlFieldMetadata.isContent() )
@@ -472,7 +472,7 @@ public class XdocGenerator
      */
     private String getModelXmlDescriptor( ModelClass rootModelClass )
     {
-        return getElementXmlDescriptor( rootModelClass, null, new Stack() );
+        return getElementXmlDescriptor( rootModelClass, null, new Stack<String>() );
     }
 
     /**
@@ -484,7 +484,7 @@ public class XdocGenerator
      * @return the String representing the tree model
      * @throws ModelloRuntimeException
      */
-    private String getElementXmlDescriptor( ModelClass modelClass, ModelAssociation association, Stack stack )
+    private String getElementXmlDescriptor( ModelClass modelClass, ModelAssociation association, Stack<String> stack )
         throws ModelloRuntimeException
     {
         StringBuffer sb = new StringBuffer();
@@ -493,9 +493,38 @@ public class XdocGenerator
 
         String tagName = resolveTagName( modelClass, association );
 
+        // <tagName
         sb.append( "&lt;<a href=\"#" ).append( getAnchorName( tagName ) ).append( "\">" );
         sb.append( tagName ).append( "</a>" );
 
+        boolean addNewline = false;
+        if ( stack.size() == 0 )
+        {
+            // try to add XML Schema reference
+            try
+            {
+                String targetNamespace = XsdModelHelper.getTargetNamespace( modelClass.getModel(), getGeneratedVersion() );
+
+                XmlModelMetadata xmlModelMetadata = (XmlModelMetadata) modelClass.getModel().getMetadata( XmlModelMetadata.ID );
+
+                if ( StringUtils.isNotBlank( targetNamespace ) && ( xmlModelMetadata.getSchemaLocation() != null ) )
+                {
+                    String schemaLocation = xmlModelMetadata.getSchemaLocation( getGeneratedVersion() );
+
+                    sb.append( " xmlns=\"" + targetNamespace + "\"" );
+                    sb.append( " xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n" );
+                    sb.append( "  xsi:schemaLocation=\"" + targetNamespace );
+                    sb.append( " <a href=\"" + schemaLocation + "\">" + schemaLocation + "</a>\"" );
+
+                    addNewline = true;
+                }
+            }
+            catch ( ModelloException me )
+            {
+                // ignore unavailable XML Schema configuration
+            }
+        }
+
         String id = tagName + '/' + modelClass.getPackageName() + '.' + modelClass.getName();
         if ( stack.contains( id ) )
         {
@@ -504,20 +533,27 @@ public class XdocGenerator
             return sb.toString();
         }
 
-        List attributeFields = getAttributeFieldsForClass( modelClass );
+        List<ModelField> fields = getFieldsForXml( modelClass, getGeneratedVersion() );
 
-        attributeFields = getNonTransientFields( attributeFields );
+        List<ModelField> attributeFields = getXmlAttributeFields( fields);
 
         if ( attributeFields.size() > 0 )
         {
 
-            for ( Iterator iter = attributeFields.iterator(); iter.hasNext(); )
+            for ( ModelField f : attributeFields )
             {
-                ModelField f = (ModelField) iter.next();
-
                 XmlFieldMetadata xmlFieldMetadata = (XmlFieldMetadata) f.getMetadata( XmlFieldMetadata.ID );
 
-                sb.append( ' ' );
+                if ( addNewline )
+                {
+                    addNewline = false;
+
+                    sb.append( "\n  " );
+                }
+                else
+                {
+                    sb.append( ' ' );
+                }
 
                 sb.append( resolveTagName( f, xmlFieldMetadata ) ).append( "=.." );
             }
@@ -526,11 +562,8 @@ public class XdocGenerator
 
         }
 
-        List fields = getFieldsForClass( modelClass );
         fields.removeAll( attributeFields );
 
-        fields = getNonTransientFields( fields );
-
         if ( ( fields.size() == 0 ) || ( ( fields.size() == 1 ) && hasContentField( fields ) ) )
         {
             sb.append( "/&gt;\n" );
@@ -541,10 +574,8 @@ public class XdocGenerator
 
             stack.push( id );
 
-            for ( Iterator iter = fields.iterator(); iter.hasNext(); )
+            for ( ModelField f : fields )
             {
-                ModelField f = (ModelField) iter.next();
-
                 XmlFieldMetadata xmlFieldMetadata = (XmlFieldMetadata) f.getMetadata( XmlFieldMetadata.ID );
 
                 XdocFieldMetadata xdocFieldMetadata = (XdocFieldMetadata) f.getMetadata( XdocFieldMetadata.ID );
