@@ -34,7 +34,6 @@ import org.codehaus.modello.plugin.java.javasource.JMethod;
 import org.codehaus.modello.plugin.java.javasource.JParameter;
 import org.codehaus.modello.plugin.java.javasource.JSourceCode;
 import org.codehaus.modello.plugin.java.javasource.JSourceWriter;
-import org.codehaus.modello.plugin.java.metadata.JavaClassMetadata;
 import org.codehaus.modello.plugin.java.metadata.JavaFieldMetadata;
 import org.codehaus.modello.plugin.model.ModelClassMetadata;
 import org.codehaus.modello.plugins.xml.metadata.XmlAssociationMetadata;
@@ -42,14 +41,13 @@ import org.codehaus.modello.plugins.xml.metadata.XmlFieldMetadata;
 import org.codehaus.modello.plugins.xml.metadata.XmlModelMetadata;
 
 import java.io.IOException;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Properties;
 
 /**
  * @author <a href="mailto:jason@modello.org">Jason van Zyl </a>
  * @author <a href="mailto:evenisse@codehaus.org">Emmanuel Venisse </a>
- * @version $Id: Xpp3WriterGenerator.java 1311 2009-08-20 23:28:35Z hboutemy $
+ * @version $Id: Xpp3WriterGenerator.java 1474 2010-04-24 19:59:00Z bentmann $
  */
 public class Xpp3WriterGenerator
     extends AbstractXpp3Generator
@@ -87,6 +85,7 @@ public class Xpp3WriterGenerator
 
         jClass.addImport( "org.codehaus.plexus.util.xml.pull.XmlSerializer" );
         jClass.addImport( "org.codehaus.plexus.util.xml.pull.MXSerializer" );
+        jClass.addImport( "java.io.OutputStream" );
         jClass.addImport( "java.io.Writer" );
         jClass.addImport( "java.text.DateFormat" );
         jClass.addImport( "java.util.Iterator" );
@@ -106,7 +105,9 @@ public class Xpp3WriterGenerator
 
         String rootElement = resolveTagName( rootClass );
 
-        // Write the write method which will do the marshalling.
+        // ----------------------------------------------------------------------
+        // Write the write( Writer, Model ) method which will do the unmarshalling.
+        // ----------------------------------------------------------------------
 
         JMethod marshall = new JMethod( "write" );
 
@@ -136,6 +137,37 @@ public class Xpp3WriterGenerator
 
         jClass.addMethod( marshall );
 
+        // ----------------------------------------------------------------------
+        // Write the write( OutputStream, Model ) method which will do the unmarshalling.
+        // ----------------------------------------------------------------------
+
+        marshall = new JMethod( "write" );
+
+        marshall.addParameter( new JParameter( new JClass( "OutputStream" ), "stream" ) );
+        marshall.addParameter( new JParameter( new JClass( root ), rootElementParameterName ) );
+
+        marshall.addException( new JClass( "java.io.IOException" ) );
+
+        sc = marshall.getSourceCode();
+
+        sc.add( "XmlSerializer serializer = new MXSerializer();" );
+
+        sc.add(
+            "serializer.setProperty( \"http://xmlpull.org/v1/doc/properties.html#serializer-indentation\", \"  \" );" );
+
+        sc.add(
+            "serializer.setProperty( \"http://xmlpull.org/v1/doc/properties.html#serializer-line-separator\", \"\\n\" );" );
+
+        sc.add( "serializer.setOutput( stream, " + rootElementParameterName + ".getModelEncoding() );" );
+
+        sc.add( "serializer.startDocument( " + rootElementParameterName + ".getModelEncoding(), null );" );
+
+        sc.add( "write" + root + "( " + rootElementParameterName + ", \"" + rootElement + "\", serializer );" );
+
+        sc.add( "serializer.endDocument();" );
+
+        jClass.addMethod( marshall );
+
         writeAllClasses( objectModel, jClass );
 
         jClass.print( sourceWriter );
@@ -146,18 +178,8 @@ public class Xpp3WriterGenerator
     private void writeAllClasses( Model objectModel, JClass jClass )
         throws ModelloException
     {
-        for ( Iterator i = objectModel.getClasses( getGeneratedVersion() ).iterator(); i.hasNext(); )
+        for ( ModelClass clazz : getClasses( objectModel ) )
         {
-            ModelClass clazz = (ModelClass) i.next();
-
-            JavaClassMetadata javaClassMetadata = (JavaClassMetadata) clazz.getMetadata( JavaClassMetadata.ID );
-
-            if ( !javaClassMetadata.isEnabled() )
-            {
-                // Skip import of those classes that are not enabled for the java plugin.
-                continue;
-            }
-
             writeClass( clazz, jClass );
         }
     }
@@ -181,11 +203,6 @@ public class Xpp3WriterGenerator
 
         JSourceCode sc = marshall.getSourceCode();
 
-        sc.add( "if ( " + uncapClassName + " != null )" );
-
-        sc.add( "{" );
-        sc.indent();
-
         ModelClassMetadata classMetadata = (ModelClassMetadata) modelClass.getMetadata( ModelClassMetadata.ID );
 
         String namespace = null;
@@ -217,15 +234,11 @@ public class Xpp3WriterGenerator
 
         String contentValue = null;
 
-        List modelFields = modelClass.getAllFields( getGeneratedVersion(), true );
-
-        modelFields = getNonTransientFields( modelFields );
+        List<ModelField> modelFields = getFieldsForXml( modelClass, getGeneratedVersion() );
 
         // XML attributes
-        for ( Iterator i = modelFields.iterator(); i.hasNext(); )
+        for ( ModelField field : modelFields )
         {
-            ModelField field = (ModelField) i.next();
-
             XmlFieldMetadata xmlFieldMetadata = (XmlFieldMetadata) field.getMetadata( XmlFieldMetadata.ID );
 
             JavaFieldMetadata javaFieldMetadata = (JavaFieldMetadata) field.getMetadata( JavaFieldMetadata.ID );
@@ -262,11 +275,8 @@ public class Xpp3WriterGenerator
         }
 
         // XML tags
-        for ( Iterator fieldIterator = modelFields.iterator(); fieldIterator.hasNext(); )
+        for ( ModelField field : modelFields )
         {
-
-            ModelField field = (ModelField) fieldIterator.next();
-
             XmlFieldMetadata xmlFieldMetadata = (XmlFieldMetadata) field.getMetadata( XmlFieldMetadata.ID );
 
             if ( xmlFieldMetadata.isContent() )
@@ -435,9 +445,6 @@ public class Xpp3WriterGenerator
 
         sc.add( "serializer.endTag( NAMESPACE, tagName );" );
 
-        sc.unindent();
-        sc.add( "}" );
-
         jClass.addMethod( marshall );
     }
 }
