@@ -29,11 +29,9 @@ import org.codehaus.modello.model.ModelAssociation;
 import org.codehaus.modello.model.ModelClass;
 import org.codehaus.modello.model.ModelField;
 import org.codehaus.modello.plugin.xsd.metadata.XsdClassMetadata;
-import org.codehaus.modello.plugin.xsd.metadata.XsdModelMetadata;
 import org.codehaus.modello.plugins.xml.AbstractXmlGenerator;
 import org.codehaus.modello.plugins.xml.metadata.XmlAssociationMetadata;
 import org.codehaus.modello.plugins.xml.metadata.XmlFieldMetadata;
-import org.codehaus.modello.plugins.xml.metadata.XmlModelMetadata;
 import org.codehaus.plexus.util.StringUtils;
 import org.codehaus.plexus.util.WriterFactory;
 import org.codehaus.plexus.util.xml.PrettyPrintXMLWriter;
@@ -43,14 +41,13 @@ import java.io.File;
 import java.io.IOException;
 import java.io.Writer;
 import java.util.HashSet;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Properties;
 import java.util.Set;
 
 /**
  * @author <a href="mailto:brett@codehaus.org">Brett Porter</a>
- * @version $Id: XsdGenerator.java 1274 2009-07-04 19:39:46Z hboutemy $
+ * @version $Id: XsdGenerator.java 1484 2010-05-08 17:25:54Z bentmann $
  */
 public class XsdGenerator
     extends AbstractXmlGenerator
@@ -115,39 +112,11 @@ public class XsdGenerator
             ModelClass root = objectModel.getClass( objectModel.getRoot( getGeneratedVersion() ),
                                                     getGeneratedVersion() );
 
-            XmlModelMetadata xmlModelMetadata = (XmlModelMetadata) root.getModel().getMetadata( XmlModelMetadata.ID );
-
-            XsdModelMetadata xsdModelMetadata = (XsdModelMetadata) root.getModel().getMetadata( XsdModelMetadata.ID );
-
-            String namespace;
-            if ( StringUtils.isNotEmpty( xsdModelMetadata.getNamespace() ) )
-            {
-                namespace = xsdModelMetadata.getNamespace( getGeneratedVersion() );
-            }
-            else
-            {
-                // xsd.namespace is not set, try using xml.namespace
-                if ( StringUtils.isEmpty( xmlModelMetadata.getNamespace() ) )
-                {
-                    throw new ModelloException( "Cannot generate xsd without xmlns specification:"
-                                                + " <model xml.namespace='...'> or <model xsd.namespace='...'>" );
-                }
-
-                namespace = xmlModelMetadata.getNamespace( getGeneratedVersion() );
-            }
+            String namespace = XsdModelHelper.getNamespace( root.getModel(), getGeneratedVersion() );
 
             w.addAttribute( "xmlns", namespace );
 
-            String targetNamespace;
-            if ( xsdModelMetadata.getTargetNamespace() == null )
-            {
-                // xsd.target-namespace not set, using namespace
-                targetNamespace = namespace;
-            }
-            else
-            {
-                targetNamespace = xsdModelMetadata.getTargetNamespace( getGeneratedVersion() );
-            }
+            String targetNamespace = XsdModelHelper.getTargetNamespace( root.getModel(), getGeneratedVersion(), namespace );
 
             // add targetNamespace if attribute is not blank (specifically set to avoid a target namespace)
             if ( StringUtils.isNotBlank( targetNamespace ) )
@@ -167,7 +136,7 @@ public class XsdGenerator
             // Element descriptors
             // Traverse from root so "abstract" models aren't included
             int initialCapacity = objectModel.getClasses( getGeneratedVersion() ).size();
-            writeComplexTypeDescriptor( w, objectModel, root, new HashSet( initialCapacity ) );
+            writeComplexTypeDescriptor( w, objectModel, root, new HashSet<ModelClass>( initialCapacity ) );
 
             w.endElement();
         }
@@ -213,51 +182,44 @@ public class XsdGenerator
         }
     }
 
-    private void writeComplexTypeDescriptor( XMLWriter w, Model objectModel, ModelClass modelClass, Set written )
+    private void writeComplexTypeDescriptor( XMLWriter w, Model objectModel, ModelClass modelClass,
+                                             Set<ModelClass> written )
     {
         written.add( modelClass );
 
         w.startElement( "xs:complexType" );
         w.addAttribute( "name", modelClass.getName() );
 
-        List fields = getFieldsForClass( modelClass );
+        List<ModelField> fields = getFieldsForXml( modelClass, getGeneratedVersion() );
 
-        fields = getNonTransientFields( fields );
+        ModelField contentField = getContentField( fields );
 
-        boolean hasContentField = hasContentField( fields );
+        boolean hasContentField = contentField != null;
 
-        List attributeFields = getAttributeFieldsForClass( modelClass );
+        List<ModelField> attributeFields = getXmlAttributeFields( fields );
 
         fields.removeAll( attributeFields );
 
-        boolean mixedContent = hasContentField && fields.size() > 0;
-
-        // other fields with complexType
-        // if yes it's a mixed content element and attribute
-        if ( mixedContent )
-        {
-            w.addAttribute( "mixed", "true" );
-        }
-        else if ( hasContentField )
+        if ( hasContentField )
         {
             // yes it's only an extension of xs:string
             w.startElement( "xs:simpleContent" );
 
             w.startElement( "xs:extension" );
 
-            w.addAttribute( "base", "xs:string" );
+            w.addAttribute( "base", getXsdType( contentField.getType() ) );
         }
 
         writeClassDocumentation( w, modelClass );
 
-        Set toWrite = new HashSet();
+        Set<ModelClass> toWrite = new HashSet<ModelClass>();
 
         if ( fields.size() > 0 )
         {
             XsdClassMetadata xsdClassMetadata = (XsdClassMetadata) modelClass.getMetadata( XsdClassMetadata.ID );
             boolean compositorAll = XsdClassMetadata.COMPOSITOR_ALL.equals( xsdClassMetadata.getCompositor() );
 
-            if ( ( mixedContent ) || ( !hasContentField ) )
+            if ( !hasContentField )
             {
                 if ( compositorAll )
                 {
@@ -268,10 +230,9 @@ public class XsdGenerator
                     w.startElement( "xs:sequence" );
                 }
             }
-            for ( Iterator j = fields.iterator(); j.hasNext(); )
-            {
-                ModelField field = (ModelField) j.next();
 
+            for ( ModelField field : fields )
+            {
                 XmlFieldMetadata xmlFieldMetadata = (XmlFieldMetadata) field.getMetadata( XmlFieldMetadata.ID );
 
                 String fieldTagName = resolveTagName( field, xmlFieldMetadata );
@@ -296,7 +257,7 @@ public class XsdGenerator
                 {
                     // nothing to add
                 }
-                else if ( ( xsdType != null ) || "char".equals( field.getType() ) || "Char".equals( field.getType() ) )
+                else if ( ( xsdType != null ) || "char".equals( field.getType() ) || "Character".equals( field.getType() ) )
                 {
                     w.addAttribute( "name", fieldTagName );
 
@@ -351,23 +312,12 @@ public class XsdGenerator
                                         + "then class " + modelClass.getName() + " MUST be declared as xsd.compositor=\"sequence\"" );
                                 }
 
-                                if ( mixedContent )
-                                {
-                                    w.startElement( "xs:element" );
-                                    w.addAttribute( "minOccurs", "0" );
-                                }
-
                                 w.addAttribute( "name", resolveTagName( fieldTagName, xmlAssociationMetadata ) );
 
                                 w.addAttribute( "type", fieldModelClass.getName() );
                                 w.addAttribute( "maxOccurs", "unbounded" );
 
                                 writeFieldDocumentation( w, field );
-
-                                if ( mixedContent )
-                                {
-                                    w.endElement();
-                                }
                             }
                         }
                         else
@@ -413,16 +363,15 @@ public class XsdGenerator
                     w.endElement();
                 }
             }// end fields iterator
-            if ( !hasContentField || mixedContent )
+
+            if ( !hasContentField )
             {
                 w.endElement(); // xs:all or xs:sequence
             }
         }
 
-        for ( Iterator j = attributeFields.iterator(); j.hasNext(); )
+        for ( ModelField field : attributeFields )
         {
-            ModelField field = (ModelField) j.next();
-
             XmlFieldMetadata xmlFieldMetadata = (XmlFieldMetadata) field.getMetadata( XmlFieldMetadata.ID );
 
             w.startElement( "xs:attribute" );
@@ -445,7 +394,7 @@ public class XsdGenerator
 
             writeFieldDocumentation( w, field );
 
-            if ( "char".equals( field.getType() ) || "Char".equals( field.getType() ) )
+            if ( "char".equals( field.getType() ) || "Character".equals( field.getType() ) )
             {
                 writeCharElement( w );
             }
@@ -458,7 +407,7 @@ public class XsdGenerator
             w.endElement();
         }
 
-        if ( hasContentField && !mixedContent )
+        if ( hasContentField )
         {
             w.endElement(); //xs:extension
 
@@ -468,9 +417,8 @@ public class XsdGenerator
 
         w.endElement(); // xs:complexType
 
-        for ( Iterator iter = toWrite.iterator(); iter.hasNext(); )
+        for ( ModelClass fieldModelClass : toWrite )
         {
-            ModelClass fieldModelClass = (ModelClass) iter.next();
             if ( !written.contains( fieldModelClass ) )
             {
                 writeComplexTypeDescriptor( w, objectModel, fieldModelClass, written );
@@ -565,11 +513,11 @@ public class XsdGenerator
         {
             return "xs:long";
         }
-        else if ("float".equals( type ) || "Float".equals( type ) )
+        else if ( "float".equals( type ) || "Float".equals( type ) )
         {
             return "xs:float";
         }
-        else if ("double".equals( type ) || "Double".equals( type ) )
+        else if ( "double".equals( type ) || "Double".equals( type ) )
         {
             return "xs:double";
         }
